SPOOKYSWAP_MASTERCHEF_V3_ABI = [{"inputs": [
    {"internalType": "contract IMasterChef", "name": "_MASTER_CHEF", "type": "address"},
    {"internalType": "contract IERC20", "name": "_boo", "type": "address"},
    {"internalType": "uint256", "name": "_MASTER_PID", "type": "uint256"}], "stateMutability": "nonpayable",
                                 "type": "constructor"}, {"anonymous": False, "inputs": [
    {"indexed": True, "internalType": "address", "name": "by", "type": "address"},
    {"indexed": True, "internalType": "address", "name": "to", "type": "address"}], "name": "AddAuth", "type": "event"},
                                {"anonymous": False, "inputs": [
                                    {"indexed": True, "internalType": "address", "name": "user", "type": "address"},
                                    {"indexed": True, "internalType": "uint256", "name": "pid", "type": "uint256"},
                                    {"indexed": False, "internalType": "uint256", "name": "amount", "type": "uint256"},
                                    {"indexed": True, "internalType": "address", "name": "to", "type": "address"}],
                                 "name": "Deposit", "type": "event"}, {"anonymous": False, "inputs": [
        {"indexed": True, "internalType": "address", "name": "user", "type": "address"},
        {"indexed": True, "internalType": "uint256", "name": "pid", "type": "uint256"},
        {"indexed": False, "internalType": "uint256", "name": "amount", "type": "uint256"},
        {"indexed": True, "internalType": "address", "name": "to", "type": "address"}], "name": "EmergencyWithdraw",
                                                                       "type": "event"}, {"anonymous": False,
                                                                                          "inputs": [{"indexed": True,
                                                                                                      "internalType": "address",
                                                                                                      "name": "user",
                                                                                                      "type": "address"},
                                                                                                     {"indexed": True,
                                                                                                      "internalType": "uint256",
                                                                                                      "name": "pid",
                                                                                                      "type": "uint256"},
                                                                                                     {"indexed": False,
                                                                                                      "internalType": "uint256",
                                                                                                      "name": "amount",
                                                                                                      "type": "uint256"}],
                                                                                          "name": "Harvest",
                                                                                          "type": "event"},
                                {"anonymous": False, "inputs": [], "name": "LogInit", "type": "event"},
                                {"anonymous": False, "inputs": [
                                    {"indexed": True, "internalType": "uint256", "name": "pid", "type": "uint256"},
                                    {"indexed": False, "internalType": "uint256", "name": "allocPoint",
                                     "type": "uint256"},
                                    {"indexed": True, "internalType": "contract IERC20", "name": "lpToken",
                                     "type": "address"},
                                    {"indexed": False, "internalType": "contract IRewarder", "name": "rewarder",
                                     "type": "address"},
                                    {"indexed": False, "internalType": "bool", "name": "update", "type": "bool"}],
                                 "name": "LogPoolAddition", "type": "event"}, {"anonymous": False, "inputs": [
        {"indexed": True, "internalType": "uint256", "name": "pid", "type": "uint256"},
        {"indexed": False, "internalType": "uint256", "name": "allocPoint", "type": "uint256"},
        {"indexed": False, "internalType": "contract IRewarder", "name": "rewarder", "type": "address"},
        {"indexed": False, "internalType": "bool", "name": "overwrite", "type": "bool"},
        {"indexed": False, "internalType": "bool", "name": "update", "type": "bool"}], "name": "LogSetPool",
                                                                               "type": "event"}, {"anonymous": False,
                                                                                                  "inputs": [
                                                                                                      {"indexed": True,
                                                                                                       "internalType": "uint256",
                                                                                                       "name": "pid",
                                                                                                       "type": "uint256"},
                                                                                                      {"indexed": False,
                                                                                                       "internalType": "uint256",
                                                                                                       "name": "lastRewardTime",
                                                                                                       "type": "uint256"},
                                                                                                      {"indexed": False,
                                                                                                       "internalType": "uint256",
                                                                                                       "name": "lpSupply",
                                                                                                       "type": "uint256"},
                                                                                                      {"indexed": False,
                                                                                                       "internalType": "uint256",
                                                                                                       "name": "accBooPerShare",
                                                                                                       "type": "uint256"}],
                                                                                                  "name": "LogUpdatePool",
                                                                                                  "type": "event"},
                                {"anonymous": False, "inputs": [
                                    {"indexed": True, "internalType": "address", "name": "by", "type": "address"},
                                    {"indexed": True, "internalType": "address", "name": "to", "type": "address"}],
                                 "name": "RevokeAuth", "type": "event"}, {"anonymous": False, "inputs": [
        {"indexed": True, "internalType": "address", "name": "by", "type": "address"},
        {"indexed": True, "internalType": "address", "name": "to", "type": "address"}], "name": "SetAdmin",
                                                                          "type": "event"}, {"anonymous": False,
                                                                                             "inputs": [
                                                                                                 {"indexed": True,
                                                                                                  "internalType": "address",
                                                                                                  "name": "user",
                                                                                                  "type": "address"},
                                                                                                 {"indexed": True,
                                                                                                  "internalType": "uint256",
                                                                                                  "name": "pid",
                                                                                                  "type": "uint256"},
                                                                                                 {"indexed": False,
                                                                                                  "internalType": "uint256",
                                                                                                  "name": "amount",
                                                                                                  "type": "uint256"},
                                                                                                 {"indexed": True,
                                                                                                  "internalType": "address",
                                                                                                  "name": "to",
                                                                                                  "type": "address"}],
                                                                                             "name": "Withdraw",
                                                                                             "type": "event"},
                                {"inputs": [], "name": "BOO",
                                 "outputs": [{"internalType": "contract IERC20", "name": "", "type": "address"}],
                                 "stateMutability": "view", "type": "function"}, {"inputs": [], "name": "MASTER_CHEF",
                                                                                  "outputs": [{
                                                                                                  "internalType": "contract IMasterChef",
                                                                                                  "name": "",
                                                                                                  "type": "address"}],
                                                                                  "stateMutability": "view",
                                                                                  "type": "function"},
                                {"inputs": [], "name": "MASTER_PID",
                                 "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                 "stateMutability": "view", "type": "function"},
                                {"inputs": [], "name": "V1_HARVEST_QUERY_TIME",
                                 "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                 "stateMutability": "view", "type": "function"}, {
                                    "inputs": [{"internalType": "uint64", "name": "allocPoint", "type": "uint64"},
                                               {"internalType": "contract IERC20", "name": "_lpToken",
                                                "type": "address"},
                                               {"internalType": "contract IRewarder", "name": "_rewarder",
                                                "type": "address"},
                                               {"internalType": "bool", "name": "update", "type": "bool"}],
                                    "name": "add", "outputs": [], "stateMutability": "nonpayable", "type": "function"},
                                {"inputs": [{"internalType": "address", "name": "_auth", "type": "address"}],
                                 "name": "addAuth", "outputs": [], "stateMutability": "nonpayable", "type": "function"},
                                {"inputs": [], "name": "admin",
                                 "outputs": [{"internalType": "address", "name": "", "type": "address"}],
                                 "stateMutability": "view", "type": "function"},
                                {"inputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                 "name": "authorized",
                                 "outputs": [{"internalType": "address", "name": "", "type": "address"}],
                                 "stateMutability": "view", "type": "function"}, {"inputs": [], "name": "booPerSecond",
                                                                                  "outputs": [
                                                                                      {"internalType": "uint256",
                                                                                       "name": "amount",
                                                                                       "type": "uint256"}],
                                                                                  "stateMutability": "view",
                                                                                  "type": "function"}, {
                                    "inputs": [{"internalType": "uint256", "name": "pid", "type": "uint256"},
                                               {"internalType": "uint256", "name": "amount", "type": "uint256"},
                                               {"internalType": "address", "name": "to", "type": "address"}],
                                    "name": "deposit", "outputs": [], "stateMutability": "nonpayable",
                                    "type": "function"}, {
                                    "inputs": [{"internalType": "uint256", "name": "pid", "type": "uint256"},
                                               {"internalType": "uint256", "name": "amount", "type": "uint256"}],
                                    "name": "deposit", "outputs": [], "stateMutability": "nonpayable",
                                    "type": "function"}, {
                                    "inputs": [{"internalType": "uint256", "name": "pid", "type": "uint256"},
                                               {"internalType": "address", "name": "to", "type": "address"}],
                                    "name": "emergencyWithdraw", "outputs": [], "stateMutability": "nonpayable",
                                    "type": "function"},
                                {"inputs": [{"internalType": "uint256", "name": "pid", "type": "uint256"}],
                                 "name": "emergencyWithdraw", "outputs": [], "stateMutability": "nonpayable",
                                 "type": "function"},
                                {"inputs": [{"internalType": "uint256", "name": "pid", "type": "uint256"}],
                                 "name": "getFarmData", "outputs": [{"components": [
                                    {"internalType": "uint128", "name": "accBooPerShare", "type": "uint128"},
                                    {"internalType": "uint64", "name": "lastRewardTime", "type": "uint64"},
                                    {"internalType": "uint64", "name": "allocPoint", "type": "uint64"}],
                                                                     "internalType": "struct MasterChefV2.PoolInfo",
                                                                     "name": "", "type": "tuple"},
                                                                    {"internalType": "uint256", "name": "",
                                                                     "type": "uint256"},
                                                                    {"internalType": "contract IRewarder", "name": "",
                                                                     "type": "address"}], "stateMutability": "view",
                                 "type": "function"},
                                {"inputs": [], "name": "harvestAll", "outputs": [], "stateMutability": "nonpayable",
                                 "type": "function"}, {"inputs": [], "name": "harvestFromMasterChef", "outputs": [],
                                                       "stateMutability": "nonpayable", "type": "function"},
                                {"inputs": [{"internalType": "uint256[]", "name": "pids", "type": "uint256[]"}],
                                 "name": "harvestMultiple", "outputs": [], "stateMutability": "nonpayable",
                                 "type": "function"}, {"inputs": [
        {"internalType": "contract IERC20", "name": "dummyToken", "type": "address"}], "name": "init", "outputs": [],
                                                       "stateMutability": "nonpayable", "type": "function"},
                                {"inputs": [{"internalType": "address", "name": "", "type": "address"}],
                                 "name": "isAuth", "outputs": [{"internalType": "bool", "name": "", "type": "bool"}],
                                 "stateMutability": "view", "type": "function"},
                                {"inputs": [{"internalType": "contract IERC20", "name": "", "type": "address"}],
                                 "name": "isLpToken", "outputs": [{"internalType": "bool", "name": "", "type": "bool"}],
                                 "stateMutability": "view", "type": "function"},
                                {"inputs": [], "name": "lastV1HarvestTimestamp",
                                 "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                 "stateMutability": "view", "type": "function"},
                                {"inputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                 "name": "lpSupplies",
                                 "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                 "stateMutability": "view", "type": "function"},
                                {"inputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                 "name": "lpToken",
                                 "outputs": [{"internalType": "contract IERC20", "name": "", "type": "address"}],
                                 "stateMutability": "view", "type": "function"},
                                {"inputs": [], "name": "massUpdateAllPools", "outputs": [],
                                 "stateMutability": "nonpayable", "type": "function"},
                                {"inputs": [{"internalType": "uint256[]", "name": "pids", "type": "uint256[]"}],
                                 "name": "massUpdatePools", "outputs": [], "stateMutability": "nonpayable",
                                 "type": "function"},
                                {"inputs": [{"internalType": "bytes[]", "name": "data", "type": "bytes[]"}],
                                 "name": "multicall",
                                 "outputs": [{"internalType": "bytes[]", "name": "results", "type": "bytes[]"}],
                                 "stateMutability": "payable", "type": "function"}, {
                                    "inputs": [{"internalType": "uint256", "name": "_pid", "type": "uint256"},
                                               {"internalType": "address", "name": "_user", "type": "address"}],
                                    "name": "pendingBOO",
                                    "outputs": [{"internalType": "uint256", "name": "pending", "type": "uint256"}],
                                    "stateMutability": "view", "type": "function"},
                                {"inputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                 "name": "poolInfo",
                                 "outputs": [{"internalType": "uint128", "name": "accBooPerShare", "type": "uint128"},
                                             {"internalType": "uint64", "name": "lastRewardTime", "type": "uint64"},
                                             {"internalType": "uint64", "name": "allocPoint", "type": "uint64"}],
                                 "stateMutability": "view", "type": "function"},
                                {"inputs": [], "name": "poolInfoAmount",
                                 "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                 "stateMutability": "view", "type": "function"}, {"inputs": [], "name": "poolLength",
                                                                                  "outputs": [
                                                                                      {"internalType": "uint256",
                                                                                       "name": "pools",
                                                                                       "type": "uint256"}],
                                                                                  "stateMutability": "view",
                                                                                  "type": "function"},
                                {"inputs": [], "name": "queryHarvestFromMasterChef", "outputs": [],
                                 "stateMutability": "nonpayable", "type": "function"},
                                {"inputs": [{"internalType": "address", "name": "_auth", "type": "address"}],
                                 "name": "revokeAuth", "outputs": [], "stateMutability": "nonpayable",
                                 "type": "function"},
                                {"inputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                 "name": "rewarder",
                                 "outputs": [{"internalType": "contract IRewarder", "name": "", "type": "address"}],
                                 "stateMutability": "view", "type": "function"}, {
                                    "inputs": [{"internalType": "address", "name": "token", "type": "address"},
                                               {"internalType": "uint256", "name": "value", "type": "uint256"},
                                               {"internalType": "uint256", "name": "deadline", "type": "uint256"},
                                               {"internalType": "uint8", "name": "v", "type": "uint8"},
                                               {"internalType": "bytes32", "name": "r", "type": "bytes32"},
                                               {"internalType": "bytes32", "name": "s", "type": "bytes32"}],
                                    "name": "selfPermit", "outputs": [], "stateMutability": "payable",
                                    "type": "function"}, {
                                    "inputs": [{"internalType": "address", "name": "token", "type": "address"},
                                               {"internalType": "uint256", "name": "nonce", "type": "uint256"},
                                               {"internalType": "uint256", "name": "expiry", "type": "uint256"},
                                               {"internalType": "uint8", "name": "v", "type": "uint8"},
                                               {"internalType": "bytes32", "name": "r", "type": "bytes32"},
                                               {"internalType": "bytes32", "name": "s", "type": "bytes32"}],
                                    "name": "selfPermitAllowed", "outputs": [], "stateMutability": "payable",
                                    "type": "function"}, {
                                    "inputs": [{"internalType": "address", "name": "token", "type": "address"},
                                               {"internalType": "uint256", "name": "nonce", "type": "uint256"},
                                               {"internalType": "uint256", "name": "expiry", "type": "uint256"},
                                               {"internalType": "uint8", "name": "v", "type": "uint8"},
                                               {"internalType": "bytes32", "name": "r", "type": "bytes32"},
                                               {"internalType": "bytes32", "name": "s", "type": "bytes32"}],
                                    "name": "selfPermitAllowedIfNecessary", "outputs": [], "stateMutability": "payable",
                                    "type": "function"}, {
                                    "inputs": [{"internalType": "address", "name": "token", "type": "address"},
                                               {"internalType": "uint256", "name": "value", "type": "uint256"},
                                               {"internalType": "uint256", "name": "deadline", "type": "uint256"},
                                               {"internalType": "uint8", "name": "v", "type": "uint8"},
                                               {"internalType": "bytes32", "name": "r", "type": "bytes32"},
                                               {"internalType": "bytes32", "name": "s", "type": "bytes32"}],
                                    "name": "selfPermitIfNecessary", "outputs": [], "stateMutability": "payable",
                                    "type": "function"}, {
                                    "inputs": [{"internalType": "uint256", "name": "_pid", "type": "uint256"},
                                               {"internalType": "uint64", "name": "_allocPoint", "type": "uint64"},
                                               {"internalType": "contract IRewarder", "name": "_rewarder",
                                                "type": "address"},
                                               {"internalType": "bool", "name": "overwrite", "type": "bool"},
                                               {"internalType": "bool", "name": "update", "type": "bool"}],
                                    "name": "set", "outputs": [], "stateMutability": "nonpayable", "type": "function"},
                                {"inputs": [{"internalType": "address", "name": "newAdmin", "type": "address"}],
                                 "name": "setAdmin", "outputs": [], "stateMutability": "nonpayable",
                                 "type": "function"}, {
                                    "inputs": [{"internalType": "uint256[]", "name": "_pid", "type": "uint256[]"},
                                               {"internalType": "uint64[]", "name": "_allocPoint", "type": "uint64[]"},
                                               {"internalType": "contract IRewarder[]", "name": "_rewarders",
                                                "type": "address[]"},
                                               {"internalType": "bool[]", "name": "overwrite", "type": "bool[]"},
                                               {"internalType": "bool", "name": "update", "type": "bool"}],
                                    "name": "setBatch", "outputs": [], "stateMutability": "nonpayable",
                                    "type": "function"}, {
                                    "inputs": [{"internalType": "uint256[]", "name": "_pid", "type": "uint256[]"},
                                               {"internalType": "uint64[]", "name": "_allocPoint", "type": "uint64[]"},
                                               {"internalType": "bool", "name": "update", "type": "bool"}],
                                    "name": "setBatchSlim", "outputs": [], "stateMutability": "nonpayable",
                                    "type": "function"}, {
                                    "inputs": [{"internalType": "uint256", "name": "newTime", "type": "uint256"},
                                               {"internalType": "bool", "name": "inDays", "type": "bool"}],
                                    "name": "setV1HarvestQueryTime", "outputs": [], "stateMutability": "nonpayable",
                                    "type": "function"},
                                {"inputs": [{"internalType": "address", "name": "token", "type": "address"}],
                                 "name": "supportsPermits",
                                 "outputs": [{"internalType": "bytes32", "name": "domainSeparator", "type": "bytes32"}],
                                 "stateMutability": "view", "type": "function"},
                                {"inputs": [], "name": "totalAllocPoint",
                                 "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                 "stateMutability": "view", "type": "function"},
                                {"inputs": [{"internalType": "uint256", "name": "pid", "type": "uint256"}],
                                 "name": "updatePool", "outputs": [{"components": [
                                    {"internalType": "uint128", "name": "accBooPerShare", "type": "uint128"},
                                    {"internalType": "uint64", "name": "lastRewardTime", "type": "uint64"},
                                    {"internalType": "uint64", "name": "allocPoint", "type": "uint64"}],
                                                                    "internalType": "struct MasterChefV2.PoolInfo",
                                                                    "name": "pool", "type": "tuple"}],
                                 "stateMutability": "nonpayable", "type": "function"}, {
                                    "inputs": [{"internalType": "uint256", "name": "", "type": "uint256"},
                                               {"internalType": "address", "name": "", "type": "address"}],
                                    "name": "userInfo",
                                    "outputs": [{"internalType": "uint256", "name": "amount", "type": "uint256"},
                                                {"internalType": "uint256", "name": "rewardDebt", "type": "uint256"}],
                                    "stateMutability": "view", "type": "function"}, {
                                    "inputs": [{"internalType": "uint256", "name": "pid", "type": "uint256"},
                                               {"internalType": "uint256", "name": "amount", "type": "uint256"},
                                               {"internalType": "address", "name": "to", "type": "address"}],
                                    "name": "withdraw", "outputs": [], "stateMutability": "nonpayable",
                                    "type": "function"}, {
                                    "inputs": [{"internalType": "uint256", "name": "pid", "type": "uint256"},
                                               {"internalType": "uint256", "name": "amount", "type": "uint256"}],
                                    "name": "withdraw", "outputs": [], "stateMutability": "nonpayable",
                                    "type": "function"}]




import json

POOL_ADDRESSES_PROVIDER = json.loads("""
[
    {
      "inputs": [
        {
          "internalType": "string",
          "name": "marketId",
          "type": "string"
        },
        {
          "internalType": "address",
          "name": "owner",
          "type": "address"
        }
      ],
      "stateMutability": "nonpayable",
      "type": "constructor"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "oldAddress",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "ACLAdminUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "oldAddress",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "ACLManagerUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "bytes32",
          "name": "id",
          "type": "bytes32"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "oldAddress",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "AddressSet",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "bytes32",
          "name": "id",
          "type": "bytes32"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "proxyAddress",
          "type": "address"
        },
        {
          "indexed": false,
          "internalType": "address",
          "name": "oldImplementationAddress",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newImplementationAddress",
          "type": "address"
        }
      ],
      "name": "AddressSetAsProxy",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "string",
          "name": "oldMarketId",
          "type": "string"
        },
        {
          "indexed": true,
          "internalType": "string",
          "name": "newMarketId",
          "type": "string"
        }
      ],
      "name": "MarketIdSet",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "previousOwner",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newOwner",
          "type": "address"
        }
      ],
      "name": "OwnershipTransferred",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "oldAddress",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "PoolConfiguratorUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "oldAddress",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "PoolDataProviderUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "oldAddress",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "PoolUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "oldAddress",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "PriceOracleSentinelUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "oldAddress",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "PriceOracleUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "bytes32",
          "name": "id",
          "type": "bytes32"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "proxyAddress",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "implementationAddress",
          "type": "address"
        }
      ],
      "name": "ProxyCreated",
      "type": "event"
    },
    {
      "inputs": [],
      "name": "getACLAdmin",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getACLManager",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "bytes32",
          "name": "id",
          "type": "bytes32"
        }
      ],
      "name": "getAddress",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getMarketId",
      "outputs": [
        {
          "internalType": "string",
          "name": "",
          "type": "string"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getPool",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getPoolConfigurator",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getPoolDataProvider",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getPriceOracle",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getPriceOracleSentinel",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "owner",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "renounceOwnership",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "newAclAdmin",
          "type": "address"
        }
      ],
      "name": "setACLAdmin",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "newAclManager",
          "type": "address"
        }
      ],
      "name": "setACLManager",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "bytes32",
          "name": "id",
          "type": "bytes32"
        },
        {
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "setAddress",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "bytes32",
          "name": "id",
          "type": "bytes32"
        },
        {
          "internalType": "address",
          "name": "newImplementationAddress",
          "type": "address"
        }
      ],
      "name": "setAddressAsProxy",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "string",
          "name": "newMarketId",
          "type": "string"
        }
      ],
      "name": "setMarketId",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "newPoolConfiguratorImpl",
          "type": "address"
        }
      ],
      "name": "setPoolConfiguratorImpl",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "newDataProvider",
          "type": "address"
        }
      ],
      "name": "setPoolDataProvider",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "newPoolImpl",
          "type": "address"
        }
      ],
      "name": "setPoolImpl",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "newPriceOracle",
          "type": "address"
        }
      ],
      "name": "setPriceOracle",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "newPriceOracleSentinel",
          "type": "address"
        }
      ],
      "name": "setPriceOracleSentinel",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "newOwner",
          "type": "address"
        }
      ],
      "name": "transferOwnership",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    }
]
""")
        


import json

ALPACA_VAULT_ABI = json.loads('''
[
    {
        "anonymous": false,
        "inputs": [
            {
                "indexed": true,
                "internalType": "uint256",
                "name": "id",
                "type": "uint256"
            },
            {
                "indexed": false,
                "internalType": "uint256",
                "name": "amount",
                "type": "uint256"
            },
            {
                "indexed": false,
                "internalType": "uint256",
                "name": "healthBefore",
                "type": "uint256"
            },
            {
                "indexed": false,
                "internalType": "uint256",
                "name": "healthAfter",
                "type": "uint256"
            }
        ],
        "name": "AddCollateral",
        "type": "event"
    },
    {
        "anonymous": false,
        "inputs": [
            {
                "indexed": true,
                "internalType": "uint256",
                "name": "id",
                "type": "uint256"
            },
            {
                "indexed": false,
                "internalType": "uint256",
                "name": "debtShare",
                "type": "uint256"
            }
        ],
        "name": "AddDebt",
        "type": "event"
    },
    {
        "anonymous": false,
        "inputs": [
            {
                "indexed": true,
                "internalType": "address",
                "name": "owner",
                "type": "address"
            },
            {
                "indexed": true,
                "internalType": "address",
                "name": "spender",
                "type": "address"
            },
            {
                "indexed": false,
                "internalType": "uint256",
                "name": "value",
                "type": "uint256"
            }
        ],
        "name": "Approval",
        "type": "event"
    },
    {
        "anonymous": false,
        "inputs": [
            {
                "indexed": true,
                "internalType": "uint256",
                "name": "id",
                "type": "uint256"
            },
            {
                "indexed": true,
                "internalType": "address",
                "name": "killer",
                "type": "address"
            },
            {
                "indexed": false,
                "internalType": "address",
                "name": "owner",
                "type": "address"
            },
            {
                "indexed": false,
                "internalType": "uint256",
                "name": "posVal",
                "type": "uint256"
            },
            {
                "indexed": false,
                "internalType": "uint256",
                "name": "debt",
                "type": "uint256"
            },
            {
                "indexed": false,
                "internalType": "uint256",
                "name": "prize",
                "type": "uint256"
            },
            {
                "indexed": false,
                "internalType": "uint256",
                "name": "left",
                "type": "uint256"
            }
        ],
        "name": "Kill",
        "type": "event"
    },
    {
        "anonymous": false,
        "inputs": [
            {
                "indexed": true,
                "internalType": "address",
                "name": "previousOwner",
                "type": "address"
            },
            {
                "indexed": true,
                "internalType": "address",
                "name": "newOwner",
                "type": "address"
            }
        ],
        "name": "OwnershipTransferred",
        "type": "event"
    },
    {
        "anonymous": false,
        "inputs": [
            {
                "indexed": true,
                "internalType": "uint256",
                "name": "id",
                "type": "uint256"
            },
            {
                "indexed": false,
                "internalType": "uint256",
                "name": "debtShare",
                "type": "uint256"
            }
        ],
        "name": "RemoveDebt",
        "type": "event"
    },
    {
        "anonymous": false,
        "inputs": [
            {
                "indexed": true,
                "internalType": "address",
                "name": "from",
                "type": "address"
            },
            {
                "indexed": true,
                "internalType": "address",
                "name": "to",
                "type": "address"
            },
            {
                "indexed": false,
                "internalType": "uint256",
                "name": "value",
                "type": "uint256"
            }
        ],
        "name": "Transfer",
        "type": "event"
    },
    {
        "anonymous": false,
        "inputs": [
            {
                "indexed": true,
                "internalType": "uint256",
                "name": "id",
                "type": "uint256"
            },
            {
                "indexed": false,
                "internalType": "uint256",
                "name": "loan",
                "type": "uint256"
            }
        ],
        "name": "Work",
        "type": "event"
    },
    {
        "inputs": [],
        "name": "POSITION_ID",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "STRATEGY",
        "outputs": [
            {
                "internalType": "address",
                "name": "",
                "type": "address"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "_IN_EXEC_LOCK",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "id",
                "type": "uint256"
            },
            {
                "internalType": "uint256",
                "name": "amount",
                "type": "uint256"
            },
            {
                "internalType": "bool",
                "name": "goRogue",
                "type": "bool"
            },
            {
                "internalType": "bytes",
                "name": "data",
                "type": "bytes"
            }
        ],
        "name": "addCollateral",
        "outputs": [],
        "stateMutability": "payable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "owner",
                "type": "address"
            },
            {
                "internalType": "address",
                "name": "spender",
                "type": "address"
            }
        ],
        "name": "allowance",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "spender",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "amount",
                "type": "uint256"
            }
        ],
        "name": "approve",
        "outputs": [
            {
                "internalType": "bool",
                "name": "",
                "type": "bool"
            }
        ],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "account",
                "type": "address"
            }
        ],
        "name": "balanceOf",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "config",
        "outputs": [
            {
                "internalType": "contract IVaultConfig",
                "name": "",
                "type": "address"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "debtShare",
                "type": "uint256"
            }
        ],
        "name": "debtShareToVal",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "debtToken",
        "outputs": [
            {
                "internalType": "address",
                "name": "",
                "type": "address"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "debtVal",
                "type": "uint256"
            }
        ],
        "name": "debtValToShare",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "decimals",
        "outputs": [
            {
                "internalType": "uint8",
                "name": "",
                "type": "uint8"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "spender",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "subtractedValue",
                "type": "uint256"
            }
        ],
        "name": "decreaseAllowance",
        "outputs": [
            {
                "internalType": "bool",
                "name": "",
                "type": "bool"
            }
        ],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "amountToken",
                "type": "uint256"
            }
        ],
        "name": "deposit",
        "outputs": [],
        "stateMutability": "payable",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "fairLaunchPoolId",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "spender",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "addedValue",
                "type": "uint256"
            }
        ],
        "name": "increaseAllowance",
        "outputs": [
            {
                "internalType": "bool",
                "name": "",
                "type": "bool"
            }
        ],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "contract IVaultConfig",
                "name": "_config",
                "type": "address"
            },
            {
                "internalType": "address",
                "name": "_token",
                "type": "address"
            },
            {
                "internalType": "string",
                "name": "_name",
                "type": "string"
            },
            {
                "internalType": "string",
                "name": "_symbol",
                "type": "string"
            },
            {
                "internalType": "uint8",
                "name": "_decimals",
                "type": "uint8"
            },
            {
                "internalType": "address",
                "name": "_debtToken",
                "type": "address"
            }
        ],
        "name": "initialize",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "id",
                "type": "uint256"
            }
        ],
        "name": "kill",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "lastAccrueTime",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "name",
        "outputs": [
            {
                "internalType": "string",
                "name": "",
                "type": "string"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "nextPositionID",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "owner",
        "outputs": [
            {
                "internalType": "address",
                "name": "",
                "type": "address"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "value",
                "type": "uint256"
            }
        ],
        "name": "pendingInterest",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "id",
                "type": "uint256"
            }
        ],
        "name": "positionInfo",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            },
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "name": "positions",
        "outputs": [
            {
                "internalType": "address",
                "name": "worker",
                "type": "address"
            },
            {
                "internalType": "address",
                "name": "owner",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "debtShare",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "value",
                "type": "uint256"
            }
        ],
        "name": "reduceReserve",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "renounceOwnership",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "targetedToken",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "amount",
                "type": "uint256"
            }
        ],
        "name": "requestFunds",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "reservePool",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "_poolId",
                "type": "uint256"
            }
        ],
        "name": "setFairLaunchPoolId",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "symbol",
        "outputs": [
            {
                "internalType": "string",
                "name": "",
                "type": "string"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "token",
        "outputs": [
            {
                "internalType": "address",
                "name": "",
                "type": "address"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "totalSupply",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "totalToken",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "recipient",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "amount",
                "type": "uint256"
            }
        ],
        "name": "transfer",
        "outputs": [
            {
                "internalType": "bool",
                "name": "",
                "type": "bool"
            }
        ],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "sender",
                "type": "address"
            },
            {
                "internalType": "address",
                "name": "recipient",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "amount",
                "type": "uint256"
            }
        ],
        "name": "transferFrom",
        "outputs": [
            {
                "internalType": "bool",
                "name": "",
                "type": "bool"
            }
        ],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "newOwner",
                "type": "address"
            }
        ],
        "name": "transferOwnership",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "contract IVaultConfig",
                "name": "_config",
                "type": "address"
            }
        ],
        "name": "updateConfig",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "vaultDebtShare",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "vaultDebtVal",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "share",
                "type": "uint256"
            }
        ],
        "name": "withdraw",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "to",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "value",
                "type": "uint256"
            }
        ],
        "name": "withdrawReserve",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "id",
                "type": "uint256"
            },
            {
                "internalType": "address",
                "name": "worker",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "principalAmount",
                "type": "uint256"
            },
            {
                "internalType": "uint256",
                "name": "borrowAmount",
                "type": "uint256"
            },
            {
                "internalType": "uint256",
                "name": "maxReturn",
                "type": "uint256"
            },
            {
                "internalType": "bytes",
                "name": "data",
                "type": "bytes"
            }
        ],
        "name": "work",
        "outputs": [],
        "stateMutability": "payable",
        "type": "function"
    },
    {
        "stateMutability": "payable",
        "type": "receive"
    }
]
''')




import json

ACCESS_CONTROL_MANAGER = json.loads("""
[
  {
    "inputs": [],
    "stateMutability": "nonpayable",
    "type": "constructor"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "contractAddress",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "string",
        "name": "functionSig",
        "type": "string"
      }
    ],
    "name": "PermissionGranted",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "contractAddress",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "string",
        "name": "functionSig",
        "type": "string"
      }
    ],
    "name": "PermissionRevoked",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "previousAdminRole",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "newAdminRole",
        "type": "bytes32"
      }
    ],
    "name": "RoleAdminChanged",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "sender",
        "type": "address"
      }
    ],
    "name": "RoleGranted",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "sender",
        "type": "address"
      }
    ],
    "name": "RoleRevoked",
    "type": "event"
  },
  {
    "inputs": [],
    "name": "DEFAULT_ADMIN_ROLE",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      }
    ],
    "name": "getRoleAdmin",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "contractAddress",
        "type": "address"
      },
      {
        "internalType": "string",
        "name": "functionSig",
        "type": "string"
      },
      {
        "internalType": "address",
        "name": "accountToPermit",
        "type": "address"
      }
    ],
    "name": "giveCallPermission",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "grantRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "contractAddress",
        "type": "address"
      },
      {
        "internalType": "string",
        "name": "functionSig",
        "type": "string"
      }
    ],
    "name": "hasPermission",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "hasRole",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "internalType": "string",
        "name": "functionSig",
        "type": "string"
      }
    ],
    "name": "isAllowedToCall",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "renounceRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "contractAddress",
        "type": "address"
      },
      {
        "internalType": "string",
        "name": "functionSig",
        "type": "string"
      },
      {
        "internalType": "address",
        "name": "accountToRevoke",
        "type": "address"
      }
    ],
    "name": "revokeCallPermission",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "revokeRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes4",
        "name": "interfaceId",
        "type": "bytes4"
      }
    ],
    "name": "supportsInterface",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  }
]
""")
        



import json

LENDING_POOL_ADDRESSES_PROVIDER = json.loads("""
[
    {
      "inputs": [
        {
          "internalType": "string",
          "name": "marketId",
          "type": "string"
        }
      ],
      "stateMutability": "nonpayable",
      "type": "constructor"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "bytes32",
          "name": "id",
          "type": "bytes32"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        },
        {
          "indexed": false,
          "internalType": "bool",
          "name": "hasProxy",
          "type": "bool"
        }
      ],
      "name": "AddressSet",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "ConfigurationAdminUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "EmergencyAdminUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "LendingPoolCollateralManagerUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "LendingPoolConfiguratorUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "LendingPoolUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "LendingRateOracleUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "string",
          "name": "newMarketId",
          "type": "string"
        }
      ],
      "name": "MarketIdSet",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "previousOwner",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newOwner",
          "type": "address"
        }
      ],
      "name": "OwnershipTransferred",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "PriceOracleUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "bytes32",
          "name": "id",
          "type": "bytes32"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "ProxyCreated",
      "type": "event"
    },
    {
      "inputs": [
        {
          "internalType": "bytes32",
          "name": "id",
          "type": "bytes32"
        }
      ],
      "name": "getAddress",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getEmergencyAdmin",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getLendingPool",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getLendingPoolCollateralManager",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getLendingPoolConfigurator",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getLendingRateOracle",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getMarketId",
      "outputs": [
        {
          "internalType": "string",
          "name": "",
          "type": "string"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getPoolAdmin",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getPriceOracle",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "owner",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "renounceOwnership",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "bytes32",
          "name": "id",
          "type": "bytes32"
        },
        {
          "internalType": "address",
          "name": "newAddress",
          "type": "address"
        }
      ],
      "name": "setAddress",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "bytes32",
          "name": "id",
          "type": "bytes32"
        },
        {
          "internalType": "address",
          "name": "implementationAddress",
          "type": "address"
        }
      ],
      "name": "setAddressAsProxy",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "emergencyAdmin",
          "type": "address"
        }
      ],
      "name": "setEmergencyAdmin",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "manager",
          "type": "address"
        }
      ],
      "name": "setLendingPoolCollateralManager",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "configurator",
          "type": "address"
        }
      ],
      "name": "setLendingPoolConfiguratorImpl",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "pool",
          "type": "address"
        }
      ],
      "name": "setLendingPoolImpl",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "lendingRateOracle",
          "type": "address"
        }
      ],
      "name": "setLendingRateOracle",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "string",
          "name": "marketId",
          "type": "string"
        }
      ],
      "name": "setMarketId",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "admin",
          "type": "address"
        }
      ],
      "name": "setPoolAdmin",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "priceOracle",
          "type": "address"
        }
      ],
      "name": "setPriceOracle",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "newOwner",
          "type": "address"
        }
      ],
      "name": "transferOwnership",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    }
]
""")
        


import json

LIDO_ORACLE_DAEMON_CONFIG_ABI = json.loads('''
[
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_admin",
        "type": "address"
      },
      {
        "internalType": "address[]",
        "name": "_configManagers",
        "type": "address[]"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "constructor"
  },
  {
    "inputs": [
      {
        "internalType": "string",
        "name": "key",
        "type": "string"
      }
    ],
    "name": "EmptyValue",
    "type": "error"
  },
  {
    "inputs": [
      {
        "internalType": "string",
        "name": "key",
        "type": "string"
      }
    ],
    "name": "ValueDoesntExist",
    "type": "error"
  },
  {
    "inputs": [
      {
        "internalType": "string",
        "name": "key",
        "type": "string"
      }
    ],
    "name": "ValueExists",
    "type": "error"
  },
  {
    "inputs": [
      {
        "internalType": "string",
        "name": "key",
        "type": "string"
      },
      {
        "internalType": "bytes",
        "name": "value",
        "type": "bytes"
      }
    ],
    "name": "ValueIsSame",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "ZeroAddress",
    "type": "error"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "string",
        "name": "key",
        "type": "string"
      },
      {
        "indexed": false,
        "internalType": "bytes",
        "name": "value",
        "type": "bytes"
      }
    ],
    "name": "ConfigValueSet",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "string",
        "name": "key",
        "type": "string"
      }
    ],
    "name": "ConfigValueUnset",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "string",
        "name": "key",
        "type": "string"
      },
      {
        "indexed": false,
        "internalType": "bytes",
        "name": "value",
        "type": "bytes"
      }
    ],
    "name": "ConfigValueUpdated",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "previousAdminRole",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "newAdminRole",
        "type": "bytes32"
      }
    ],
    "name": "RoleAdminChanged",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "sender",
        "type": "address"
      }
    ],
    "name": "RoleGranted",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "sender",
        "type": "address"
      }
    ],
    "name": "RoleRevoked",
    "type": "event"
  },
  {
    "inputs": [],
    "name": "CONFIG_MANAGER_ROLE",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "DEFAULT_ADMIN_ROLE",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "string",
        "name": "_key",
        "type": "string"
      }
    ],
    "name": "get",
    "outputs": [
      {
        "internalType": "bytes",
        "name": "",
        "type": "bytes"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "string[]",
        "name": "_keys",
        "type": "string[]"
      }
    ],
    "name": "getList",
    "outputs": [
      {
        "internalType": "bytes[]",
        "name": "",
        "type": "bytes[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      }
    ],
    "name": "getRoleAdmin",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "uint256",
        "name": "index",
        "type": "uint256"
      }
    ],
    "name": "getRoleMember",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      }
    ],
    "name": "getRoleMemberCount",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "grantRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "hasRole",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "renounceRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "revokeRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "string",
        "name": "_key",
        "type": "string"
      },
      {
        "internalType": "bytes",
        "name": "_value",
        "type": "bytes"
      }
    ],
    "name": "set",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes4",
        "name": "interfaceId",
        "type": "bytes4"
      }
    ],
    "name": "supportsInterface",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "string",
        "name": "_key",
        "type": "string"
      }
    ],
    "name": "unset",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "string",
        "name": "_key",
        "type": "string"
      },
      {
        "internalType": "bytes",
        "name": "_value",
        "type": "bytes"
      }
    ],
    "name": "update",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  }
]
''')



import json

UNISWAP_REPAY_ADAPTER = json.loads("""
[
    {
      "inputs": [
        {
          "internalType": "contract ILendingPoolAddressesProvider",
          "name": "addressesProvider",
          "type": "address"
        },
        {
          "internalType": "contract IUniswapV2Router02",
          "name": "uniswapRouter",
          "type": "address"
        },
        {
          "internalType": "address",
          "name": "wethAddress",
          "type": "address"
        }
      ],
      "stateMutability": "nonpayable",
      "type": "constructor"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "previousOwner",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newOwner",
          "type": "address"
        }
      ],
      "name": "OwnershipTransferred",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "address",
          "name": "fromAsset",
          "type": "address"
        },
        {
          "indexed": false,
          "internalType": "address",
          "name": "toAsset",
          "type": "address"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "fromAmount",
          "type": "uint256"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "receivedAmount",
          "type": "uint256"
        }
      ],
      "name": "Swapped",
      "type": "event"
    },
    {
      "inputs": [],
      "name": "ADDRESSES_PROVIDER",
      "outputs": [
        {
          "internalType": "contract ILendingPoolAddressesProvider",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "FLASHLOAN_PREMIUM_TOTAL",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "LENDING_POOL",
      "outputs": [
        {
          "internalType": "contract ILendingPool",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "MAX_SLIPPAGE_PERCENT",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "ORACLE",
      "outputs": [
        {
          "internalType": "contract IPriceOracleGetter",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "UNISWAP_ROUTER",
      "outputs": [
        {
          "internalType": "contract IUniswapV2Router02",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "USD_ADDRESS",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "WETH_ADDRESS",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address[]",
          "name": "assets",
          "type": "address[]"
        },
        {
          "internalType": "uint256[]",
          "name": "amounts",
          "type": "uint256[]"
        },
        {
          "internalType": "uint256[]",
          "name": "premiums",
          "type": "uint256[]"
        },
        {
          "internalType": "address",
          "name": "initiator",
          "type": "address"
        },
        {
          "internalType": "bytes",
          "name": "params",
          "type": "bytes"
        }
      ],
      "name": "executeOperation",
      "outputs": [
        {
          "internalType": "bool",
          "name": "",
          "type": "bool"
        }
      ],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "uint256",
          "name": "amountOut",
          "type": "uint256"
        },
        {
          "internalType": "address",
          "name": "reserveIn",
          "type": "address"
        },
        {
          "internalType": "address",
          "name": "reserveOut",
          "type": "address"
        }
      ],
      "name": "getAmountsIn",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        },
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        },
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        },
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        },
        {
          "internalType": "address[]",
          "name": "",
          "type": "address[]"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "uint256",
          "name": "amountIn",
          "type": "uint256"
        },
        {
          "internalType": "address",
          "name": "reserveIn",
          "type": "address"
        },
        {
          "internalType": "address",
          "name": "reserveOut",
          "type": "address"
        }
      ],
      "name": "getAmountsOut",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        },
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        },
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        },
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        },
        {
          "internalType": "address[]",
          "name": "",
          "type": "address[]"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "owner",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "renounceOwnership",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "contract IERC20",
          "name": "token",
          "type": "address"
        }
      ],
      "name": "rescueTokens",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "collateralAsset",
          "type": "address"
        },
        {
          "internalType": "address",
          "name": "debtAsset",
          "type": "address"
        },
        {
          "internalType": "uint256",
          "name": "collateralAmount",
          "type": "uint256"
        },
        {
          "internalType": "uint256",
          "name": "debtRepayAmount",
          "type": "uint256"
        },
        {
          "internalType": "uint256",
          "name": "debtRateMode",
          "type": "uint256"
        },
        {
          "components": [
            {
              "internalType": "uint256",
              "name": "amount",
              "type": "uint256"
            },
            {
              "internalType": "uint256",
              "name": "deadline",
              "type": "uint256"
            },
            {
              "internalType": "uint8",
              "name": "v",
              "type": "uint8"
            },
            {
              "internalType": "bytes32",
              "name": "r",
              "type": "bytes32"
            },
            {
              "internalType": "bytes32",
              "name": "s",
              "type": "bytes32"
            }
          ],
          "internalType": "struct IBaseUniswapAdapter.PermitSignature",
          "name": "permitSignature",
          "type": "tuple"
        },
        {
          "internalType": "bool",
          "name": "useEthPath",
          "type": "bool"
        }
      ],
      "name": "swapAndRepay",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "newOwner",
          "type": "address"
        }
      ],
      "name": "transferOwnership",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    }
]
""")
        


import json

LIDO_CURVE_LIIQUIDITY_FARMING_MANAGER_ABI = json.loads('''
[
  {
    "outputs": [],
    "inputs": [],
    "stateMutability": "nonpayable",
    "type": "constructor"
  },
  {
    "name": "transfer_ownership",
    "outputs": [],
    "inputs": [
      {
        "type": "address",
        "name": "_to"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 36247
  },
  {
    "name": "set_rewards_contract",
    "outputs": [],
    "inputs": [
      {
        "type": "address",
        "name": "_rewards_contract"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 36277
  },
  {
    "name": "is_rewards_period_finished",
    "outputs": [
      {
        "type": "bool",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 2017
  },
  {
    "name": "start_next_rewards_period",
    "outputs": [],
    "inputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 5262
  },
  {
    "name": "recover_erc20",
    "outputs": [],
    "inputs": [
      {
        "type": "address",
        "name": "_token"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "name": "recover_erc20",
    "outputs": [],
    "inputs": [
      {
        "type": "address",
        "name": "_token"
      },
      {
        "type": "address",
        "name": "_recipient"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "name": "owner",
    "outputs": [
      {
        "type": "address",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 1241
  },
  {
    "name": "rewards_contract",
    "outputs": [
      {
        "type": "address",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 1271
  }
]
''')


import json

from defi_services.constants.query_constant import Query
from defi_services.jobs.processors.state_processor import StateProcessor
from src.defi_services.databases.mongodb_klg import MongoDB

chain_id = '0x1'

provider_url = {
    "0x38": 'https://bsc-dataseed3.binance.org/',
    '0x1': "https://rpc.ankr.com/eth",
    '0xfa': "https://fantom.publicnode.com",
    '0xa': "https://optimism.llamarpc.com",
    '0xa4b1': "https://rpc.ankr.com/arbitrum",
    '0xa86a': "https://rpc.ankr.com/avalanche",
    '0x89': "https://rpc.ankr.com/polygon"}

wallet = "0x37A745B24eefCFd230a7ca41903E701393DDFF79"
job = StateProcessor( provider_url[chain_id], chain_id)
def get_dex_processor():

    token_info=  get_token_info()

    queries = [
        {
        'query_id': 'uniswap_v3_lptokeninfo',
        "entity_id": 'uniswap_v3',
        'query_type': Query.lp_token_list,
        'number_lp': 100,
        'supplied_data' : {"token_info":list(token_info.keys())}
    }]
    res= job.run(wallet, queries, batch_size=100, max_workers=8, ignore_error=True)
    with open('lp_token_list.json','w') as f:
        json.dump(res, f)


def get_lp_token_info():
    # with open('lp_token_list.json', 'r') as f:
    #     data = json.loads(f.read())

    data = {"0x3416cf6c708da44db2624d63ea0aaef7113527c6": {
        "token0": "0xdac17f958d2ee523a2206206994597c13d831ec7",
        "token1": "0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
        "fee": "100"
      },
      "0x7858e59e0c01ea06df3af3d20ac7b0003275d4bf": {
        "token0": "0xdac17f958d2ee523a2206206994597c13d831ec7",
        "token1": "0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
        "fee": "500"
      },
      "0xee4cf3b78a74affa38c6a926282bcd8b5952818d": {
        "token0": "0xdac17f958d2ee523a2206206994597c13d831ec7",
        "token1": "0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
        "fee": "3000"
      }
    }
    queries = [
        {
            'query_id': 'uniswap_v3_lptokeninfo',
            "entity_id": 'uniswap_v3',
            'query_type':Query.lp_token_info,
            'supplied_data':  {
            'lp_token_info': data}
        },
            {
                'query_id': 'uniswap_v3_lptokenbalance',
                "entity_id": 'uniswap_v3',
                'query_type': Query.token_pair_balance,
                'supplied_data': {
            'lp_token_info': data}
            }
        ]
    res = job.run(wallet, queries, batch_size=100, max_workers=8, ignore_error=True)
    with open('lp_token_info1.json', 'w') as f:
        json.dump(res, f)

###USER
def get_user_info():
    queries = [
        {
            'query_id': 'uniswap_v3_usernft',
            "entity_id": 'uniswap_v3',
            'query_type':Query.dex_user_nft
        }
    ]

    res = job.run(wallet, queries, batch_size=100, max_workers=8, ignore_error=True)
    with open('user.json', 'w') as f:
        json.dump(res, f)

def get_user_info_v3():
    with open('user.json', 'r') as f:
        data = json.loads(f.read())
    user_data = data[0]['dex_user_nft']

    queries = [
        {
            'query_id': 'uniswap_v3_userinfo',
            "entity_id": 'uniswap_v3',
            'query_type': 'dex_user_info',
            'supplied_data': {
                'user_data': user_data
            }
            }
        ]

    res = job.run(wallet, queries, batch_size=100, max_workers=8, ignore_error=True)
    with open('user.json', 'w') as f:
        json.dump(res, f)
def get_token_info():
    mongo_klg= MongoDB()
    res={}
    tokens= mongo_klg.get_top_1000_token(chain_id)
    for token in tokens:
        token_addr = token.get("address")
        decimals = token.get("decimals")
        price = token.get("price")
        res[token_addr]= {
            'decimals': decimals,
            'price':price
            }
    return res

if __name__ == "__main__":
    get_user_info_v3()


import copy
import logging
import math
from defi_services.abis.token.erc20_abi import ERC20_ABI
from defi_services.constants.chain_constant import Chain
from defi_services.constants.entities.dex_constant import Dex
from defi_services.jobs.queriers.state_querier import StateQuerier
from defi_services.services.dex.dex_info.uniswap_info import UNISWAP_V3_ETH_INFO
from defi_services.services.dex_protocol_services import DexProtocolServices
from defi_services.utils.get_fees import get_fees
from defi_services.utils.sqrt_price_math import get_token_amount_of_user, get_token_amount_of_pool

logger = logging.getLogger("UniSwap V3 State Service")


class UniswapV3Info:
    mapping = {
        Chain.ethereum: UNISWAP_V3_ETH_INFO
    }


class UniswapV3Services(DexProtocolServices):
    def __init__(self, state_service: StateQuerier, chain_id: str = '0x1'):
        super().__init__()
        self.chain_id = chain_id
        self.state_service = state_service
        self.pool_info = UniswapV3Info.mapping.get(chain_id)
        self.web3 = self.state_service.get_w3()
        if self.pool_info is not None:
            self.factory_abi = self.pool_info.get('factory_abi')
            self.nft_token_manager_addr = self.pool_info.get('NFT_manager_address')
            self.nft_token_manager_abi = self.pool_info.get('NFT_manager_abi')
            self.factory_addr = self.pool_info.get('factory_address')

    def get_service_info(self):
        info = {
            Dex.uniswap_v3: {
                "chain_id": self.chain_id,
                "type": "dex",
                "protocol_info": self.pool_info
            }
        }
        return info

    def get_farming_supported_lp_token(self, limit: int = 10) -> dict:
        return {}

    def decode_farming_supported_lp_token(self, response_data) -> dict:
        return {}

    def get_all_supported_lp_token(self, limit: int = 100, supplied_data: dict = None):
        rpc_calls = {}
        top_token = supplied_data['token_info']
        length = min(len(top_token), limit)
        for idx0 in range(1, length):
            token0 = top_token[idx0]
            for idx1 in range(idx0 + 1, length):
                token1 = top_token[idx1]
                for fee in [100, 500, 3000, 10000]:
                    query_id = f'allPool_{self.factory_addr}_{[token0, token1, fee]}_latest'.lower()
                    rpc_calls[query_id] = self.state_service.get_function_info(
                        self.factory_addr, self.factory_abi, fn_name="getPool", fn_paras=[token0, token1, fee]
                    )
        return rpc_calls

    def decode_all_supported_lp_token(self, limit: int = 100, decoded_data: dict = None, supplied_data: dict = None):
        result = {}
        top_token = supplied_data['token_info']
        length = min(len(top_token), limit)
        for idx0 in range(1, length):
            token0 = top_token[idx0]
            for idx1 in range(idx0 + 1, length):
                token1 = top_token[idx1]
                for fee in [100, 500, 3000, 10000]:
                    query_id = f'allPool_{self.factory_addr}_{[token0, token1, fee]}_latest'.lower()
                    pool_address = decoded_data.get(query_id)
                    if pool_address != '0x0000000000000000000000000000000000000000':
                        result[pool_address] = {
                            'token0': token0,
                            'token1': token1,
                            'fee': fee
                        }

        return result

    def get_lp_token_function_info(self, supplied_data, block_number: int = "latest"):
        rpc_calls = {}
        lp_token_info = supplied_data['lp_token_info']
        for lp_token, value in lp_token_info.items():
            for fn_name in ["liquidity", "slot0", 'tickSpacing']:
                query_id = f"{fn_name}_{lp_token}_{block_number}".lower()
                rpc_calls[query_id] = self.state_service.get_function_info(
                    address=lp_token, abi=self.pool_info['pool_abi'], fn_name=fn_name, fn_paras=None,
                    block_number=block_number)

            for token_key in ['token0', 'token1']:
                token_address = value.get(token_key)
                query_id = f'decimals_{token_address}_{block_number}'.lower()
                rpc_calls[query_id] = self.state_service.get_function_info(
                    address=token_address, abi=ERC20_ABI, fn_name="decimals", block_number=block_number)
                query_id = f'{token_key}_{lp_token}_{block_number}'.lower()
                rpc_calls[query_id] = self.state_service.get_function_info(
                    address=lp_token, abi=self.pool_info['pool_abi'], fn_name=token_key, fn_paras=None,
                    block_number=block_number)

        return rpc_calls

    def decode_lp_token_info(self, supplied_data, response_data, block_number: int = "latest"):
        lp_token_info = supplied_data['lp_token_info']
        for lp_token, value in lp_token_info.items():
            liquidity_in_range = response_data.get(f'liquidity_{lp_token}_{block_number}'.lower(), 0)
            slot0 = response_data.get(f"slot0_{lp_token}_{block_number}".lower())
            token0_address = value.get('token0')
            token1_address = value.get('token1')
            if token0_address == response_data.get(f'token0_{lp_token}_{block_number}'.lower()):
                token0_decimals = response_data.get(f'decimals_{token0_address}_{block_number}'.lower())
                token1_decimals = response_data.get(f'decimals_{token1_address}_{block_number}'.lower())
            else:
                token1_decimals = response_data.get(f'decimals_{token0_address}_{block_number}'.lower())
                token0_decimals = response_data.get(f'decimals_{token1_address}_{block_number}'.lower())
            price = (slot0[0] / 2 ** 96) ** 2 * 10 ** (token0_decimals - token1_decimals)
            tick_spacing = response_data.get(f'tickSpacing_{lp_token}_{block_number}'.lower())
            lp_token_info[lp_token].update({
                "token0": token0_address,
                "token1": token1_address,
                "liquidity_in_range": liquidity_in_range,
                "price": price,
                'tick': slot0[1],
                'tick_spacing': tick_spacing,
                'token0_decimals': token0_decimals,
                'token1_decimals': token1_decimals

            })
        return lp_token_info

    def get_balance_of_token_function_info(self, supplied_data, block_number: int = "latest"):
        rpc_calls = {}
        lp_token_info = supplied_data['lp_token_info']
        for lp_token, value in lp_token_info.items():
            for token_key in ["token0", "token1"]:
                token_address = value.get(token_key, None)
                if token_address is not None:
                    balance_query_id = f'balanceOf_{token_address}_{lp_token}_{block_number}'.lower()
                    decimals_query_id = f'decimals_{token_address}_{block_number}'.lower()

                    rpc_calls[balance_query_id] = self.state_service.get_function_info(
                        address=token_address, abi=ERC20_ABI, fn_name="balanceOf", fn_paras=[lp_token],
                        block_number=block_number)

                    rpc_calls[decimals_query_id] = self.state_service.get_function_info(
                        address=token_address, abi=ERC20_ABI, fn_name="decimals", block_number=block_number)

        return rpc_calls

    def decode_balance_of_token_function_info(
            self, supplied_data, decoded_data, block_number: int = "latest"):
        lp_token_info = {}
        for lp_token, value in supplied_data['lp_token_info'].items():
            lp_token_info[lp_token] = copy.deepcopy(value)
            for token_key in ["token0", "token1"]:
                token_address = value.get(token_key, None)
                decimals = decoded_data.get(f'decimals_{token_address}_{block_number}'.lower())

                if token_address is not None:
                    balance_of = decoded_data.get(
                        f'balanceOf_{token_address}_{lp_token}_{block_number}'.lower())
                    lp_token_info[lp_token][f'{token_key}_amount'] = balance_of / 10 ** decimals

            tick = value.get('tick')
            tick_spacing = value.get('tick_spacing')
            tick_lower = (tick // tick_spacing) * tick_spacing
            tick_upper = (tick // tick_spacing + 1) * tick_spacing
            liquidity = value.get('liquidity_in_range')
            amount0, amount1 = get_token_amount_of_pool(liquidity=liquidity, tick_lower=tick_lower,
                                                        tick_upper=tick_upper)
            lp_token_info[lp_token].update({
                'token0_amount_in_range': amount0 / 10 ** value.get('token0_decimals'),
                'token1_amount_in_range': amount1 / 10 ** value.get('token1_decimals')
            })

        return lp_token_info

    ### USER
    def get_all_nft_token_of_user_function(
            self, user: str, block_number: int = "latest"):
        rpc_calls = {}
        user = self.checksum_address(user)
        self.nft_token_manager_addr = self.checksum_address(self.nft_token_manager_addr)
        nft_contract = self.web3.eth.contract(abi=self.nft_token_manager_abi,
                                              address=self.nft_token_manager_addr)
        number_token = nft_contract.functions.balanceOf(user).call()
        for idx in range(number_token):
            query_id = f'tokenOfOwnerByIndex_{self.nft_token_manager_addr}_{[user, idx]}_{block_number}'.lower()
            rpc_calls[query_id] = self.state_service.get_function_info(
                address=self.nft_token_manager_addr, abi=self.nft_token_manager_abi, fn_name="tokenOfOwnerByIndex",
                fn_paras=[user, idx],
                block_number=block_number)

        return rpc_calls

    def decode_all_nft_token_of_user_function(
            self, decode_data: dict):
        result = {}
        for query_id, token_id in decode_data.items():
            contract_addr = query_id.split("_")[1]
            if contract_addr == self.nft_token_manager_addr.lower():
                result[token_id] = {}

        return result

    def get_user_info_function(self, user: str, supplied_data: dict, stake: bool = False, block_number: int = "latest"):
        rpc_calls = {}
        user_data = supplied_data['user_data']
        for token_id, _ in user_data.items():
            query_id = f'positions_{self.nft_token_manager_addr}_{token_id}_{block_number}'.lower()
            rpc_calls[query_id] = self.state_service.get_function_info(
                address=self.nft_token_manager_addr, abi=self.nft_token_manager_abi, fn_name="positions",
                fn_paras=[int(token_id)], block_number=block_number)
        return rpc_calls

    def decode_user_info_function(self, user: str, supplied_data: dict, decoded_data: dict = None, stake: bool = False,
                                  block_number: int = "latest"):
        user_data = supplied_data['user_data']
        user_data_with_liquidity = {}
        for token_id, value in user_data.items():
            position = decoded_data.get(f'positions_{self.nft_token_manager_addr}_{token_id}_{block_number}'.lower())
            liquidity = position[7]
            if liquidity > 0:
                user_data_with_liquidity[token_id] = {
                    'token0': position[2],
                    'token1': position[3],
                    'fee': position[4],
                    'tick_lower': position[5],
                    'tick_upper': position[6],
                    'liquidity': position[7],
                    'fee_growth_inside0_x128': position[8],
                    'fee_growth_inside1_x128': position[9],
                }
        return user_data_with_liquidity

    def get_user_token_amount_function(self, user: str, supplied_data: dict, block_number: int = "latest"):
        user_data = supplied_data['user_data']
        rpc_calls = {}
        for token_id, value in user_data.items():
            token0 = value.get('token0')
            token1 = value.get('token1')
            fee = value.get('fee')
            query_id = f'getPool_{self.factory_addr}_{token_id}_{block_number}'.lower()
            rpc_calls[query_id] = self.state_service.get_function_info(
                self.factory_addr, self.factory_abi, fn_name="getPool", fn_paras=[token0, token1, fee]
            )
            for token_key in ['token0', 'token1']:
                token_address = value.get(token_key)

                query_id = f'decimals_{token_address}_{block_number}'.lower()
                rpc_calls[query_id] = self.state_service.get_function_info(
                    address=token_address, abi=ERC20_ABI, fn_name="decimals", block_number=block_number)

        return rpc_calls

    def decode_user_token_amount_function(self, user: str, supplied_data: dict, decoded_data: dict = None,
                                          block_number: int = "latest"):
        user_data = supplied_data['user_data']
        lp_token_info = supplied_data['lp_token_info']
        result = {}

        for token_id, value in user_data.items():
            lp_token_address = decoded_data.get(f'getPool_{self.factory_addr}_{token_id}_{block_number}'.lower())
            user_data[token_id].update({
                'pool_address': lp_token_address
            })
            liquidity = value.get('liquidity')
            result[token_id] = copy.deepcopy(user_data[token_id])
            token0 = value.get('token0')
            token1 = value.get('token1')
            token0_decimals = decoded_data.get(f'decimals_{token0}_{block_number}'.lower())
            token1_decimals = decoded_data.get(f'decimals_{token1}_{block_number}'.lower())
            price = lp_token_info.get(lp_token_address, {}).get("price")

            tick = lp_token_info.get(lp_token_address, {}).get('tick')
            tick_upper = value.get('tick_upper')
            tick_lower = value.get('tick_lower')
            if price and tick:
                sqrt_price_x96 = (math.sqrt(price / 10 ** (token0_decimals - token1_decimals))) * 2 ** 96

                token0_amount, token1_amount = get_token_amount_of_user(
                    liquidity=liquidity,
                    sqrt_price_x96=sqrt_price_x96,
                    tick=tick, tick_upper=tick_upper,
                    tick_lower=tick_lower)
                result[token_id].update(
                    {
                        'token0_amount': token0_amount / 10 ** token0_decimals,
                        'token1_amount': token1_amount / 10 ** token1_decimals,
                    }
                )

        return result

    def get_rewards_balance_function_info(self, user, supplied_data, block_number: int = "latest"):
        user_data = supplied_data['user_data']
        rpc_calls = {}
        for token_id, value in user_data.items():
            lp_token_address = value.get('pool_address')
            tick_lower = value.get('tick_lower')
            tick_upper = value.get('tick_upper')
            for fnc in ['feeGrowthGlobal0X128', 'feeGrowthGlobal1X128']:
                query_id = f'{fnc}_{lp_token_address}_{block_number}'.lower()
                rpc_calls[query_id] = self.state_service.get_function_info(
                    address=lp_token_address, abi=self.pool_info['pool_abi'], fn_name=fnc)

            for param in [tick_lower, tick_upper]:
                query_id = f'ticks_{lp_token_address}_{param}_{block_number}'.lower()
                rpc_calls[query_id] = self.state_service.get_function_info(
                    address=lp_token_address, abi=self.pool_info['pool_abi'], fn_name="ticks", fn_paras=[param])

        return rpc_calls

    def calculate_rewards_balance(
            self, user: str, supplied_data: dict, decoded_data: dict, block_number: int = "latest"):
        lp_token_info = supplied_data['lp_token_info']
        user_data = supplied_data['user_data']

        for token_id, value in user_data.items():
            lp_token_address = value.get('pool_address')
            tick_lower = value.get('tick_lower')
            tick_upper = value.get('tick_upper')
            fee_growth_inside_0_x128 = value.get('fee_growth_inside0_x128')
            fee_growth_inside_1_x128 = value.get('fee_growth_inside1_x128')
            fee_growth_global_0 = decoded_data.get(
                f'feeGrowthGlobal0X128_{lp_token_address}_{block_number}'.lower())
            fee_growth_global_1 = decoded_data.get(
                f'feeGrowthGlobal1X128_{lp_token_address}_{block_number}'.lower())
            fee_growth_0_low_x128 = decoded_data.get(f'ticks_{lp_token_address}_{tick_lower}_{block_number}'.lower())[2]
            fee_growth_1_low_x128 = decoded_data.get(f'ticks_{lp_token_address}_{tick_lower}_{block_number}'.lower())[3]
            fee_growth_0_hi_x128 = decoded_data.get(f'ticks_{lp_token_address}_{tick_upper}_{block_number}'.lower())[2]
            fee_growth_1_hi_x128 = decoded_data.get(f'ticks_{lp_token_address}_{tick_upper}_{block_number}'.lower())[3]
            liquidity = value.get('liquidity')
            tick = lp_token_info.get(lp_token_address, {}).get('tick')
            token0_decimals = lp_token_info.get(lp_token_address, {}).get("token0_decimals")
            token1_decimals = lp_token_info.get(lp_token_address, {}).get("token1_decimals")
            if tick and token0_decimals and token1_decimals:
                token0_reward, token1_reward = get_fees(
                    fee_growth_global_0=fee_growth_global_0,
                    fee_growth_global_1=fee_growth_global_1,
                    fee_growth_0_low=fee_growth_0_low_x128,
                    fee_growth_1_low=fee_growth_1_low_x128,
                    fee_growth_0_hi=fee_growth_0_hi_x128,
                    fee_growth_1_hi=fee_growth_1_hi_x128,
                    fee_growth_inside_0=fee_growth_inside_0_x128,
                    fee_growth_inside_1=fee_growth_inside_1_x128,
                    liquidity=liquidity, tick_lower=tick_lower,
                    tick_upper=tick_upper, tick_current=tick,
                    decimals0=token0_decimals, decimals1=token1_decimals)

                user_data[token_id].update({
                    'token0_reward': token0_reward,
                    'token1_reward': token1_reward
                })

        return user_data

    def checksum_address(self, address):
        if self.web3.is_address(address):
            address = self.web3.to_checksum_address(address)
        return address



PANCAKESWAP_V3_POOL_ABI = [{"inputs": [], "stateMutability": "nonpayable", "type": "constructor"}, {
    "anonymous": False, "inputs": [{"indexed": True, "internalType": "address", "name": "owner", "type": "address"},
                                   {"indexed": True, "internalType": "int24", "name": "tickLower", "type": "int24"},
                                   {"indexed": True, "internalType": "int24", "name": "tickUpper", "type": "int24"},
                                   {"indexed": False, "internalType": "uint128", "name": "amount", "type": "uint128"},
                                   {"indexed": False, "internalType": "uint256", "name": "amount0", "type": "uint256"},
                                   {"indexed": False, "internalType": "uint256", "name": "amount1", "type": "uint256"}],
    "name": "Burn", "type": "event"}, {
                   "anonymous": False,
                   "inputs": [{"indexed": True, "internalType": "address", "name": "owner", "type": "address"},
                              {"indexed": False, "internalType": "address", "name": "recipient", "type": "address"},
                              {"indexed": True, "internalType": "int24", "name": "tickLower", "type": "int24"},
                              {"indexed": True, "internalType": "int24", "name": "tickUpper", "type": "int24"},
                              {"indexed": False, "internalType": "uint128", "name": "amount0", "type": "uint128"},
                              {"indexed": False, "internalType": "uint128", "name": "amount1", "type": "uint128"}],
                   "name": "Collect", "type": "event"}, {
                   "anonymous": False,
                   "inputs": [{"indexed": True, "internalType": "address", "name": "sender", "type": "address"},
                              {"indexed": True, "internalType": "address", "name": "recipient", "type": "address"},
                              {"indexed": False, "internalType": "uint128", "name": "amount0", "type": "uint128"},
                              {"indexed": False, "internalType": "uint128", "name": "amount1", "type": "uint128"}],
                   "name": "CollectProtocol", "type": "event"}, {
                   "anonymous": False,
                   "inputs": [{"indexed": True, "internalType": "address", "name": "sender", "type": "address"},
                              {"indexed": True, "internalType": "address", "name": "recipient", "type": "address"},
                              {"indexed": False, "internalType": "uint256", "name": "amount0", "type": "uint256"},
                              {"indexed": False, "internalType": "uint256", "name": "amount1", "type": "uint256"},
                              {"indexed": False, "internalType": "uint256", "name": "paid0", "type": "uint256"},
                              {"indexed": False, "internalType": "uint256", "name": "paid1", "type": "uint256"}],
                   "name": "Flash", "type": "event"}, {
                   "anonymous": False, "inputs": [
        {"indexed": False, "internalType": "uint16", "name": "observationCardinalityNextOld", "type": "uint16"},
        {"indexed": False, "internalType": "uint16", "name": "observationCardinalityNextNew", "type": "uint16"}],
                   "name": "IncreaseObservationCardinalityNext", "type": "event"}, {
                   "anonymous": False,
                   "inputs": [{"indexed": False, "internalType": "uint160", "name": "sqrtPriceX96", "type": "uint160"},
                              {"indexed": False, "internalType": "int24", "name": "tick", "type": "int24"}],
                   "name": "Initialize", "type": "event"}, {
                   "anonymous": False,
                   "inputs": [{"indexed": False, "internalType": "address", "name": "sender", "type": "address"},
                              {"indexed": True, "internalType": "address", "name": "owner", "type": "address"},
                              {"indexed": True, "internalType": "int24", "name": "tickLower", "type": "int24"},
                              {"indexed": True, "internalType": "int24", "name": "tickUpper", "type": "int24"},
                              {"indexed": False, "internalType": "uint128", "name": "amount", "type": "uint128"},
                              {"indexed": False, "internalType": "uint256", "name": "amount0", "type": "uint256"},
                              {"indexed": False, "internalType": "uint256", "name": "amount1", "type": "uint256"}],
                   "name": "Mint", "type": "event"}, {
                   "anonymous": False,
                   "inputs": [{"indexed": False, "internalType": "uint32", "name": "feeProtocol0Old", "type": "uint32"},
                              {"indexed": False, "internalType": "uint32", "name": "feeProtocol1Old", "type": "uint32"},
                              {"indexed": False, "internalType": "uint32", "name": "feeProtocol0New", "type": "uint32"},
                              {
                                  "indexed": False, "internalType": "uint32", "name": "feeProtocol1New",
                                  "type": "uint32"}], "name": "SetFeeProtocol", "type": "event"}, {
                   "anonymous": False,
                   "inputs": [{"indexed": False, "internalType": "address", "name": "addr", "type": "address"}],
                   "name": "SetLmPoolEvent", "type": "event"}, {
                   "anonymous": False,
                   "inputs": [{"indexed": True, "internalType": "address", "name": "sender", "type": "address"},
                              {"indexed": True, "internalType": "address", "name": "recipient", "type": "address"},
                              {"indexed": False, "internalType": "int256", "name": "amount0", "type": "int256"},
                              {"indexed": False, "internalType": "int256", "name": "amount1", "type": "int256"},
                              {"indexed": False, "internalType": "uint160", "name": "sqrtPriceX96", "type": "uint160"},
                              {"indexed": False, "internalType": "uint128", "name": "liquidity", "type": "uint128"},
                              {"indexed": False, "internalType": "int24", "name": "tick", "type": "int24"}, {
                                  "indexed": False, "internalType": "uint128", "name": "protocolFeesToken0",
                                  "type": "uint128"}, {
                                  "indexed": False, "internalType": "uint128", "name": "protocolFeesToken1",
                                  "type": "uint128"}], "name": "Swap", "type": "event"}, {
                   "inputs": [{"internalType": "int24", "name": "tickLower", "type": "int24"},
                              {"internalType": "int24", "name": "tickUpper", "type": "int24"},
                              {"internalType": "uint128", "name": "amount", "type": "uint128"}], "name": "burn",
                   "outputs": [{"internalType": "uint256", "name": "amount0", "type": "uint256"},
                               {"internalType": "uint256", "name": "amount1", "type": "uint256"}],
                   "stateMutability": "nonpayable", "type": "function"}, {
                   "inputs": [{"internalType": "address", "name": "recipient", "type": "address"},
                              {"internalType": "int24", "name": "tickLower", "type": "int24"},
                              {"internalType": "int24", "name": "tickUpper", "type": "int24"},
                              {"internalType": "uint128", "name": "amount0Requested", "type": "uint128"},
                              {"internalType": "uint128", "name": "amount1Requested", "type": "uint128"}],
                   "name": "collect", "outputs": [{"internalType": "uint128", "name": "amount0", "type": "uint128"},
                                                  {"internalType": "uint128", "name": "amount1", "type": "uint128"}],
                   "stateMutability": "nonpayable", "type": "function"}, {
                   "inputs": [{"internalType": "address", "name": "recipient", "type": "address"},
                              {"internalType": "uint128", "name": "amount0Requested", "type": "uint128"},
                              {"internalType": "uint128", "name": "amount1Requested", "type": "uint128"}],
                   "name": "collectProtocol",
                   "outputs": [{"internalType": "uint128", "name": "amount0", "type": "uint128"},
                               {"internalType": "uint128", "name": "amount1", "type": "uint128"}],
                   "stateMutability": "nonpayable", "type": "function"}, {
                   "inputs": [], "name": "factory",
                   "outputs": [{"internalType": "address", "name": "", "type": "address"}], "stateMutability": "view",
                   "type": "function"}, {
                   "inputs": [], "name": "fee", "outputs": [{"internalType": "uint24", "name": "", "type": "uint24"}],
                   "stateMutability": "view", "type": "function"}, {
                   "inputs": [], "name": "feeGrowthGlobal0X128",
                   "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}], "stateMutability": "view",
                   "type": "function"}, {
                   "inputs": [], "name": "feeGrowthGlobal1X128",
                   "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}], "stateMutability": "view",
                   "type": "function"}, {
                   "inputs": [{"internalType": "address", "name": "recipient", "type": "address"},
                              {"internalType": "uint256", "name": "amount0", "type": "uint256"},
                              {"internalType": "uint256", "name": "amount1", "type": "uint256"},
                              {"internalType": "bytes", "name": "data", "type": "bytes"}], "name": "flash",
                   "outputs": [], "stateMutability": "nonpayable", "type": "function"}, {
                   "inputs": [{"internalType": "uint16", "name": "observationCardinalityNext", "type": "uint16"}],
                   "name": "increaseObservationCardinalityNext", "outputs": [], "stateMutability": "nonpayable",
                   "type": "function"}, {
                   "inputs": [{"internalType": "uint160", "name": "sqrtPriceX96", "type": "uint160"}],
                   "name": "initialize", "outputs": [], "stateMutability": "nonpayable", "type": "function"}, {
                   "inputs": [], "name": "liquidity",
                   "outputs": [{"internalType": "uint128", "name": "", "type": "uint128"}], "stateMutability": "view",
                   "type": "function"}, {
                   "inputs": [], "name": "lmPool",
                   "outputs": [{"internalType": "contract IPancakeV3LmPool", "name": "", "type": "address"}],
                   "stateMutability": "view", "type": "function"}, {
                   "inputs": [], "name": "maxLiquidityPerTick",
                   "outputs": [{"internalType": "uint128", "name": "", "type": "uint128"}], "stateMutability": "view",
                   "type": "function"}, {
                   "inputs": [{"internalType": "address", "name": "recipient", "type": "address"},
                              {"internalType": "int24", "name": "tickLower", "type": "int24"},
                              {"internalType": "int24", "name": "tickUpper", "type": "int24"},
                              {"internalType": "uint128", "name": "amount", "type": "uint128"},
                              {"internalType": "bytes", "name": "data", "type": "bytes"}], "name": "mint",
                   "outputs": [{"internalType": "uint256", "name": "amount0", "type": "uint256"},
                               {"internalType": "uint256", "name": "amount1", "type": "uint256"}],
                   "stateMutability": "nonpayable", "type": "function"}, {
                   "inputs": [{"internalType": "uint256", "name": "", "type": "uint256"}], "name": "observations",
                   "outputs": [{"internalType": "uint32", "name": "blockTimestamp", "type": "uint32"},
                               {"internalType": "int56", "name": "tickCumulative", "type": "int56"}, {
                                   "internalType": "uint160", "name": "secondsPerLiquidityCumulativeX128",
                                   "type": "uint160"}, {"internalType": "bool", "name": "initialized", "type": "bool"}],
                   "stateMutability": "view", "type": "function"}, {
                   "inputs": [{"internalType": "uint32[]", "name": "secondsAgos", "type": "uint32[]"}],
                   "name": "observe",
                   "outputs": [{"internalType": "int56[]", "name": "tickCumulatives", "type": "int56[]"}, {
                       "internalType": "uint160[]", "name": "secondsPerLiquidityCumulativeX128s", "type": "uint160[]"}],
                   "stateMutability": "view", "type": "function"}, {
                   "inputs": [{"internalType": "bytes32", "name": "", "type": "bytes32"}], "name": "positions",
                   "outputs": [{"internalType": "uint128", "name": "liquidity", "type": "uint128"},
                               {"internalType": "uint256", "name": "feeGrowthInside0LastX128", "type": "uint256"},
                               {"internalType": "uint256", "name": "feeGrowthInside1LastX128", "type": "uint256"},
                               {"internalType": "uint128", "name": "tokensOwed0", "type": "uint128"},
                               {"internalType": "uint128", "name": "tokensOwed1", "type": "uint128"}],
                   "stateMutability": "view", "type": "function"}, {
                   "inputs": [], "name": "protocolFees",
                   "outputs": [{"internalType": "uint128", "name": "token0", "type": "uint128"},
                               {"internalType": "uint128", "name": "token1", "type": "uint128"}],
                   "stateMutability": "view", "type": "function"}, {
                   "inputs": [{"internalType": "uint32", "name": "feeProtocol0", "type": "uint32"},
                              {"internalType": "uint32", "name": "feeProtocol1", "type": "uint32"}],
                   "name": "setFeeProtocol", "outputs": [], "stateMutability": "nonpayable", "type": "function"}, {
                   "inputs": [{"internalType": "address", "name": "_lmPool", "type": "address"}], "name": "setLmPool",
                   "outputs": [], "stateMutability": "nonpayable", "type": "function"}, {
                   "inputs": [], "name": "slot0",
                   "outputs": [{"internalType": "uint160", "name": "sqrtPriceX96", "type": "uint160"},
                               {"internalType": "int24", "name": "tick", "type": "int24"},
                               {"internalType": "uint16", "name": "observationIndex", "type": "uint16"},
                               {"internalType": "uint16", "name": "observationCardinality", "type": "uint16"},
                               {"internalType": "uint16", "name": "observationCardinalityNext", "type": "uint16"},
                               {"internalType": "uint32", "name": "feeProtocol", "type": "uint32"},
                               {"internalType": "bool", "name": "unlocked", "type": "bool"}], "stateMutability": "view",
                   "type": "function"}, {
                   "inputs": [{"internalType": "int24", "name": "tickLower", "type": "int24"},
                              {"internalType": "int24", "name": "tickUpper", "type": "int24"}],
                   "name": "snapshotCumulativesInside",
                   "outputs": [{"internalType": "int56", "name": "tickCumulativeInside", "type": "int56"},
                               {"internalType": "uint160", "name": "secondsPerLiquidityInsideX128", "type": "uint160"},
                               {"internalType": "uint32", "name": "secondsInside", "type": "uint32"}],
                   "stateMutability": "view", "type": "function"}, {
                   "inputs": [{"internalType": "address", "name": "recipient", "type": "address"},
                              {"internalType": "bool", "name": "zeroForOne", "type": "bool"},
                              {"internalType": "int256", "name": "amountSpecified", "type": "int256"},
                              {"internalType": "uint160", "name": "sqrtPriceLimitX96", "type": "uint160"},
                              {"internalType": "bytes", "name": "data", "type": "bytes"}], "name": "swap",
                   "outputs": [{"internalType": "int256", "name": "amount0", "type": "int256"},
                               {"internalType": "int256", "name": "amount1", "type": "int256"}],
                   "stateMutability": "nonpayable", "type": "function"}, {
                   "inputs": [{"internalType": "int16", "name": "", "type": "int16"}], "name": "tickBitmap",
                   "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}], "stateMutability": "view",
                   "type": "function"}, {
                   "inputs": [], "name": "tickSpacing",
                   "outputs": [{"internalType": "int24", "name": "", "type": "int24"}], "stateMutability": "view",
                   "type": "function"}, {
                   "inputs": [{"internalType": "int24", "name": "", "type": "int24"}], "name": "ticks",
                   "outputs": [{"internalType": "uint128", "name": "liquidityGross", "type": "uint128"},
                               {"internalType": "int128", "name": "liquidityNet", "type": "int128"},
                               {"internalType": "uint256", "name": "feeGrowthOutside0X128", "type": "uint256"},
                               {"internalType": "uint256", "name": "feeGrowthOutside1X128", "type": "uint256"},
                               {"internalType": "int56", "name": "tickCumulativeOutside", "type": "int56"},
                               {"internalType": "uint160", "name": "secondsPerLiquidityOutsideX128", "type": "uint160"},
                               {"internalType": "uint32", "name": "secondsOutside", "type": "uint32"},
                               {"internalType": "bool", "name": "initialized", "type": "bool"}],
                   "stateMutability": "view", "type": "function"}, {
                   "inputs": [], "name": "token0",
                   "outputs": [{"internalType": "address", "name": "", "type": "address"}], "stateMutability": "view",
                   "type": "function"}, {
                   "inputs": [], "name": "token1",
                   "outputs": [{"internalType": "address", "name": "", "type": "address"}], "stateMutability": "view",
                   "type": "function"}]



from defi_services.constants.entities.dex_services import DexServices
from defi_services.constants.entities.lending_services import LendingServices
from defi_services.constants.entities.vault_services import VaultServices
from defi_services.jobs.queriers.state_querier import StateQuerier


def init_services(state_querier: StateQuerier, chain_id: str):
    services = {}

    for protocol, value in LendingServices.mapping.get(chain_id, {}).items():
        services[protocol] = value(state_querier, chain_id)

    for protocol, value in DexServices.mapping.get(chain_id, {}).items():
        services[protocol] = value(state_querier, chain_id)

    for protocol, value in VaultServices.mapping.get(chain_id, {}).items():
        services[protocol] = value(state_querier, chain_id)

    return services



LP_TOKEN_ABI = [{"inputs": [], "payable": False, "stateMutability": "nonpayable", "type": "constructor"}, {
    "anonymous": False, "inputs": [{"indexed": True, "internalType": "address", "name": "owner", "type": "address"},
                                   {"indexed": True, "internalType": "address", "name": "spender", "type": "address"},
                                   {"indexed": False, "internalType": "uint256", "name": "value", "type": "uint256"}],
    "name": "Approval", "type": "event"}, {
                    "anonymous": False,
                    "inputs": [{"indexed": True, "internalType": "address", "name": "sender", "type": "address"},
                               {"indexed": False, "internalType": "uint256", "name": "amount0", "type": "uint256"},
                               {"indexed": False, "internalType": "uint256", "name": "amount1", "type": "uint256"},
                               {"indexed": True, "internalType": "address", "name": "to", "type": "address"}],
                    "name": "Burn", "type": "event"}, {
                    "anonymous": False,
                    "inputs": [{"indexed": True, "internalType": "address", "name": "sender", "type": "address"},
                               {"indexed": False, "internalType": "uint256", "name": "amount0", "type": "uint256"},
                               {"indexed": False, "internalType": "uint256", "name": "amount1", "type": "uint256"}],
                    "name": "Mint", "type": "event"}, {
                    "anonymous": False,
                    "inputs": [{"indexed": True, "internalType": "address", "name": "sender", "type": "address"},
                               {"indexed": False, "internalType": "uint256", "name": "amount0In", "type": "uint256"},
                               {"indexed": False, "internalType": "uint256", "name": "amount1In", "type": "uint256"},
                               {"indexed": False, "internalType": "uint256", "name": "amount0Out", "type": "uint256"},
                               {"indexed": False, "internalType": "uint256", "name": "amount1Out", "type": "uint256"},
                               {"indexed": True, "internalType": "address", "name": "to", "type": "address"}],
                    "name": "Swap", "type": "event"}, {
                    "anonymous": False,
                    "inputs": [{"indexed": False, "internalType": "uint112", "name": "reserve0", "type": "uint112"},
                               {"indexed": False, "internalType": "uint112", "name": "reserve1", "type": "uint112"}],
                    "name": "Sync", "type": "event"}, {
                    "anonymous": False,
                    "inputs": [{"indexed": True, "internalType": "address", "name": "from", "type": "address"},
                               {"indexed": True, "internalType": "address", "name": "to", "type": "address"},
                               {"indexed": False, "internalType": "uint256", "name": "value", "type": "uint256"}],
                    "name": "Transfer", "type": "event"}, {
                    "constant": True, "inputs": [], "name": "DOMAIN_SEPARATOR",
                    "outputs": [{"internalType": "bytes32", "name": "", "type": "bytes32"}], "payable": False,
                    "stateMutability": "view", "type": "function"}, {
                    "constant": True, "inputs": [], "name": "MINIMUM_LIQUIDITY",
                    "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}], "payable": False,
                    "stateMutability": "view", "type": "function"}, {
                    "constant": True, "inputs": [], "name": "PERMIT_TYPEHASH",
                    "outputs": [{"internalType": "bytes32", "name": "", "type": "bytes32"}], "payable": False,
                    "stateMutability": "view", "type": "function"}, {
                    "constant": True, "inputs": [{"internalType": "address", "name": "", "type": "address"},
                                                 {"internalType": "address", "name": "", "type": "address"}],
                    "name": "allowance", "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                    "payable": False, "stateMutability": "view", "type": "function"}, {
                    "constant": False, "inputs": [{"internalType": "address", "name": "spender", "type": "address"},
                                                  {"internalType": "uint256", "name": "value", "type": "uint256"}],
                    "name": "approve", "outputs": [{"internalType": "bool", "name": "", "type": "bool"}],
                    "payable": False, "stateMutability": "nonpayable", "type": "function"}, {
                    "constant": True, "inputs": [{"internalType": "address", "name": "", "type": "address"}],
                    "name": "balanceOf", "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                    "payable": False, "stateMutability": "view", "type": "function"}, {
                    "constant": False, "inputs": [{"internalType": "address", "name": "to", "type": "address"}],
                    "name": "burn", "outputs": [{"internalType": "uint256", "name": "amount0", "type": "uint256"},
                                                {"internalType": "uint256", "name": "amount1", "type": "uint256"}],
                    "payable": False, "stateMutability": "nonpayable", "type": "function"}, {
                    "constant": True, "inputs": [], "name": "decimals",
                    "outputs": [{"internalType": "uint8", "name": "", "type": "uint8"}], "payable": False,
                    "stateMutability": "view", "type": "function"}, {
                    "constant": True, "inputs": [], "name": "factory",
                    "outputs": [{"internalType": "address", "name": "", "type": "address"}], "payable": False,
                    "stateMutability": "view", "type": "function"}, {
                    "constant": True, "inputs": [], "name": "getReserves",
                    "outputs": [{"internalType": "uint112", "name": "_reserve0", "type": "uint112"},
                                {"internalType": "uint112", "name": "_reserve1", "type": "uint112"},
                                {"internalType": "uint32", "name": "_blockTimestampLast", "type": "uint32"}],
                    "payable": False, "stateMutability": "view", "type": "function"}, {
                    "constant": False, "inputs": [{"internalType": "address", "name": "_token0", "type": "address"},
                                                  {"internalType": "address", "name": "_token1", "type": "address"}],
                    "name": "initialize", "outputs": [], "payable": False, "stateMutability": "nonpayable",
                    "type": "function"}, {
                    "constant": True, "inputs": [], "name": "kLast",
                    "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}], "payable": False,
                    "stateMutability": "view", "type": "function"}, {
                    "constant": False, "inputs": [{"internalType": "address", "name": "to", "type": "address"}],
                    "name": "mint", "outputs": [{"internalType": "uint256", "name": "liquidity", "type": "uint256"}],
                    "payable": False, "stateMutability": "nonpayable", "type": "function"}, {
                    "constant": True, "inputs": [], "name": "name",
                    "outputs": [{"internalType": "string", "name": "", "type": "string"}], "payable": False,
                    "stateMutability": "view", "type": "function"}, {
                    "constant": True, "inputs": [{"internalType": "address", "name": "", "type": "address"}],
                    "name": "nonces", "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                    "payable": False, "stateMutability": "view", "type": "function"}, {
                    "constant": False, "inputs": [{"internalType": "address", "name": "owner", "type": "address"},
                                                  {"internalType": "address", "name": "spender", "type": "address"},
                                                  {"internalType": "uint256", "name": "value", "type": "uint256"},
                                                  {"internalType": "uint256", "name": "deadline", "type": "uint256"},
                                                  {"internalType": "uint8", "name": "v", "type": "uint8"},
                                                  {"internalType": "bytes32", "name": "r", "type": "bytes32"},
                                                  {"internalType": "bytes32", "name": "s", "type": "bytes32"}],
                    "name": "permit", "outputs": [], "payable": False, "stateMutability": "nonpayable",
                    "type": "function"}, {
                    "constant": True, "inputs": [], "name": "price0CumulativeLast",
                    "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}], "payable": False,
                    "stateMutability": "view", "type": "function"}, {
                    "constant": True, "inputs": [], "name": "price1CumulativeLast",
                    "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}], "payable": False,
                    "stateMutability": "view", "type": "function"}, {
                    "constant": False, "inputs": [{"internalType": "address", "name": "to", "type": "address"}],
                    "name": "skim", "outputs": [], "payable": False, "stateMutability": "nonpayable",
                    "type": "function"}, {
                    "constant": False, "inputs": [{"internalType": "uint256", "name": "amount0Out", "type": "uint256"},
                                                  {"internalType": "uint256", "name": "amount1Out", "type": "uint256"},
                                                  {"internalType": "address", "name": "to", "type": "address"},
                                                  {"internalType": "bytes", "name": "data", "type": "bytes"}],
                    "name": "swap", "outputs": [], "payable": False, "stateMutability": "nonpayable",
                    "type": "function"}, {
                    "constant": True, "inputs": [], "name": "symbol",
                    "outputs": [{"internalType": "string", "name": "", "type": "string"}], "payable": False,
                    "stateMutability": "view", "type": "function"}, {
                    "constant": False, "inputs": [], "name": "sync", "outputs": [], "payable": False,
                    "stateMutability": "nonpayable", "type": "function"}, {
                    "constant": True, "inputs": [], "name": "token0",
                    "outputs": [{"internalType": "address", "name": "", "type": "address"}], "payable": False,
                    "stateMutability": "view", "type": "function"}, {
                    "constant": True, "inputs": [], "name": "token1",
                    "outputs": [{"internalType": "address", "name": "", "type": "address"}], "payable": False,
                    "stateMutability": "view", "type": "function"}, {
                    "constant": True, "inputs": [], "name": "totalSupply",
                    "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}], "payable": False,
                    "stateMutability": "view", "type": "function"}, {
                    "constant": False, "inputs": [{"internalType": "address", "name": "to", "type": "address"},
                                                  {"internalType": "uint256", "name": "value", "type": "uint256"}],
                    "name": "transfer", "outputs": [{"internalType": "bool", "name": "", "type": "bool"}],
                    "payable": False, "stateMutability": "nonpayable", "type": "function"}, {
                    "constant": False, "inputs": [{"internalType": "address", "name": "from", "type": "address"},
                                                  {"internalType": "address", "name": "to", "type": "address"},
                                                  {"internalType": "uint256", "name": "value", "type": "uint256"}],
                    "name": "transferFrom", "outputs": [{"internalType": "bool", "name": "", "type": "bool"}],
                    "payable": False, "stateMutability": "nonpayable", "type": "function"}]



from defi_services.constants.token_constant import Token
from defi_services.jobs.queriers.substrate_state_querier import SubstrateStateQuerier


class SubstrateTokenServices:
    def __init__(self, state_service: SubstrateStateQuerier, chain_id: str = "polkadot"):
        self.chain_id = chain_id
        self.state_service = state_service
        self.token_info = self.get_assets()

    def get_assets(self):
        if self.chain_id in ['polkadot']:
            return {}
        client_querier = self.state_service.get_client_querier()
        assets = client_querier.query_map("Assets", "Metadata")
        result = {}
        for asset in assets.records:
            token_id = str(asset[0].value)
            result[token_id] = asset[1].value_serialized

        return result

    def get_service_info(self):
        info = {
            "token": {
                "chain_id": self.chain_id,
                "type": "token"
            }
        }
        return info

    def get_function_info(self, wallet: str, token: str, block_number: int = "latest"):
        result = self.get_function_balance_info(wallet, token, block_number)

        return result

    def get_function_balance_info(self, wallet: str, token: str = None, block_number: int = "latest"):
        if not token or token == Token.native_token:
            key = f"System_Account_{[wallet]}_{block_number}".lower()
            rpc_call = self.state_service.get_function_info("System", "Account", [wallet], block_number)
        else:
            token = int(token)
            params = [int(token), wallet]
            key = f"Assets_Account_{params}_{block_number}".lower()
            rpc_call = self.state_service.get_function_info("Assets", "Account", params, block_number)

        return {key: rpc_call}

    def get_asset_info(self, token: str):
        token = int(token)
        key = f"Assets_Asset_{token}".lower()
        rpc_call = self.state_service.get_function_info("Assets", "Asset", [token])
        return {key, rpc_call}

    def get_data(self, wallet: str, token: str = None, decoded_data: dict = None,
                 token_prices: dict = None, block_number: int = "latest"):
        if not token or token == Token.native_token:
            key = f"System_Account_{[wallet]}_{block_number}".lower()
            balance = decoded_data.get(key).get('data').get("free") / 10 ** 10
        else:
            if token not in self.token_info:
                self.token_info = self.get_assets()
            token_decimals = self.token_info.get(token, {}).get('decimals', 0)
            token = int(token)
            params = [int(token), wallet]
            key = f"Assets_Account_{params}_{block_number}".lower()
            balance = decoded_data.get(key).get("balance") / 10 ** token_decimals
        token_price = token_prices.get(token, 1)
        balance = balance * token_price
        return balance



PANCAKESWAP_MASTERCHEF_V0_ABI= [{
     "inputs": [{"internalType": "contract CakeToken", "name": "_cake", "type": "address"},
                {"internalType": "contract SyrupBar", "name": "_syrup", "type": "address"},
                {"internalType": "address", "name": "_devaddr", "type": "address"},
                {"internalType": "uint256", "name": "_cakePerBlock", "type": "uint256"},
                {"internalType": "uint256", "name": "_startBlock", "type": "uint256"}], "stateMutability": "nonpayable",
     "type": "constructor"}, {
     "anonymous": False, "inputs": [{"indexed": True, "internalType": "address", "name": "user", "type": "address"},
                                    {"indexed": True, "internalType": "uint256", "name": "pid", "type": "uint256"},
                                    {"indexed": False, "internalType": "uint256", "name": "amount", "type": "uint256"}],
     "name": "Deposit", "type": "event"}, {
     "anonymous": False, "inputs": [{"indexed": True, "internalType": "address", "name": "user", "type": "address"},
                                    {"indexed": True, "internalType": "uint256", "name": "pid", "type": "uint256"},
                                    {"indexed": False, "internalType": "uint256", "name": "amount", "type": "uint256"}],
     "name": "EmergencyWithdraw", "type": "event"}, {
     "anonymous": False,
     "inputs": [{"indexed": True, "internalType": "address", "name": "previousOwner", "type": "address"},
                {"indexed": True, "internalType": "address", "name": "newOwner", "type": "address"}],
     "name": "OwnershipTransferred", "type": "event"}, {
     "anonymous": False, "inputs": [{"indexed": True, "internalType": "address", "name": "user", "type": "address"},
                                    {"indexed": True, "internalType": "uint256", "name": "pid", "type": "uint256"},
                                    {"indexed": False, "internalType": "uint256", "name": "amount", "type": "uint256"}],
     "name": "Withdraw", "type": "event"}, {
     "inputs": [], "name": "BONUS_MULTIPLIER", "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
     "stateMutability": "view", "type": "function"}, {
     "inputs": [{"internalType": "uint256", "name": "_allocPoint", "type": "uint256"},
                {"internalType": "contract IBEP20", "name": "_lpToken", "type": "address"},
                {"internalType": "bool", "name": "_withUpdate", "type": "bool"}], "name": "add", "outputs": [],
     "stateMutability": "nonpayable", "type": "function"}, {
     "inputs": [], "name": "cake", "outputs": [{"internalType": "contract CakeToken", "name": "", "type": "address"}],
     "stateMutability": "view", "type": "function"}, {
     "inputs": [], "name": "cakePerBlock", "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
     "stateMutability": "view", "type": "function"}, {
     "inputs": [{"internalType": "uint256", "name": "_pid", "type": "uint256"},
                {"internalType": "uint256", "name": "_amount", "type": "uint256"}], "name": "deposit", "outputs": [],
     "stateMutability": "nonpayable", "type": "function"}, {
     "inputs": [{"internalType": "address", "name": "_devaddr", "type": "address"}], "name": "dev", "outputs": [],
     "stateMutability": "nonpayable", "type": "function"}, {
     "inputs": [], "name": "devaddr", "outputs": [{"internalType": "address", "name": "", "type": "address"}],
     "stateMutability": "view", "type": "function"}, {
     "inputs": [{"internalType": "uint256", "name": "_pid", "type": "uint256"}], "name": "emergencyWithdraw",
     "outputs": [], "stateMutability": "nonpayable", "type": "function"}, {
     "inputs": [{"internalType": "uint256", "name": "_amount", "type": "uint256"}], "name": "enterStaking",
     "outputs": [], "stateMutability": "nonpayable", "type": "function"}, {
     "inputs": [{"internalType": "uint256", "name": "_from", "type": "uint256"},
                {"internalType": "uint256", "name": "_to", "type": "uint256"}], "name": "getMultiplier",
     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}], "stateMutability": "view",
     "type": "function"}, {
     "inputs": [{"internalType": "uint256", "name": "_amount", "type": "uint256"}], "name": "leaveStaking",
     "outputs": [], "stateMutability": "nonpayable", "type": "function"},
 {"inputs": [], "name": "massUpdatePools", "outputs": [], "stateMutability": "nonpayable", "type": "function"}, {
     "inputs": [{"internalType": "uint256", "name": "_pid", "type": "uint256"}], "name": "migrate", "outputs": [],
     "stateMutability": "nonpayable", "type": "function"}, {
     "inputs": [], "name": "migrator",
     "outputs": [{"internalType": "contract IMigratorChef", "name": "", "type": "address"}], "stateMutability": "view",
     "type": "function"}, {
     "inputs": [], "name": "owner", "outputs": [{"internalType": "address", "name": "", "type": "address"}],
     "stateMutability": "view", "type": "function"}, {
     "inputs": [{"internalType": "uint256", "name": "_pid", "type": "uint256"},
                {"internalType": "address", "name": "_user", "type": "address"}], "name": "pendingCake",
     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}], "stateMutability": "view",
     "type": "function"}, {
     "inputs": [{"internalType": "uint256", "name": "", "type": "uint256"}], "name": "poolInfo",
     "outputs": [{"internalType": "contract IBEP20", "name": "lpToken", "type": "address"},
                 {"internalType": "uint256", "name": "allocPoint", "type": "uint256"},
                 {"internalType": "uint256", "name": "lastRewardBlock", "type": "uint256"},
                 {"internalType": "uint256", "name": "accCakePerShare", "type": "uint256"}], "stateMutability": "view",
     "type": "function"}, {
     "inputs": [], "name": "poolLength", "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
     "stateMutability": "view", "type": "function"},
 {"inputs": [], "name": "renounceOwnership", "outputs": [], "stateMutability": "nonpayable", "type": "function"}, {
     "inputs": [{"internalType": "uint256", "name": "_pid", "type": "uint256"},
                {"internalType": "uint256", "name": "_allocPoint", "type": "uint256"},
                {"internalType": "bool", "name": "_withUpdate", "type": "bool"}], "name": "set", "outputs": [],
     "stateMutability": "nonpayable", "type": "function"}, {
     "inputs": [{"internalType": "contract IMigratorChef", "name": "_migrator", "type": "address"}],
     "name": "setMigrator", "outputs": [], "stateMutability": "nonpayable", "type": "function"}, {
     "inputs": [], "name": "startBlock", "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
     "stateMutability": "view", "type": "function"}, {
     "inputs": [], "name": "syrup", "outputs": [{"internalType": "contract SyrupBar", "name": "", "type": "address"}],
     "stateMutability": "view", "type": "function"}, {
     "inputs": [], "name": "totalAllocPoint", "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
     "stateMutability": "view", "type": "function"}, {
     "inputs": [{"internalType": "address", "name": "newOwner", "type": "address"}], "name": "transferOwnership",
     "outputs": [], "stateMutability": "nonpayable", "type": "function"}, {
     "inputs": [{"internalType": "uint256", "name": "multiplierNumber", "type": "uint256"}], "name": "updateMultiplier",
     "outputs": [], "stateMutability": "nonpayable", "type": "function"}, {
     "inputs": [{"internalType": "uint256", "name": "_pid", "type": "uint256"}], "name": "updatePool", "outputs": [],
     "stateMutability": "nonpayable", "type": "function"}, {
     "inputs": [{"internalType": "uint256", "name": "", "type": "uint256"},
                {"internalType": "address", "name": "", "type": "address"}], "name": "userInfo",
     "outputs": [{"internalType": "uint256", "name": "amount", "type": "uint256"},
                 {"internalType": "uint256", "name": "rewardDebt", "type": "uint256"}], "stateMutability": "view",
     "type": "function"}, {
     "inputs": [{"internalType": "uint256", "name": "_pid", "type": "uint256"},
                {"internalType": "uint256", "name": "_amount", "type": "uint256"}], "name": "withdraw", "outputs": [],
     "stateMutability": "nonpayable", "type": "function"}]



TRAVA_VAULT_ETH = {
    "name": "Trava Vault",
    "rewardToken": "0x186d0ba3dfc3386c464eecd96a61fbb1e2da00bf",
    "reservesList": {
        "0xbf576c63c30e1ebb637138ee1c67a570ff70223d": {
            "vaultName": "rtrava",
            "tokenIn": "0x044ede67afdb0f56d7451bb3aaccaeab3f772fad",
            "metadata": {
                "vaultType": "base",
                "token": "0x044ede67afdb0f56d7451bb3aaccaeab3f772fad"
            }
        },
        "0x84d92a3d61ec158dcf878ff24ebdcf6b2b5ec1fa": {
            "vaultName": "trava",
            "tokenIn": "0x186d0ba3dfc3386c464eecd96a61fbb1e2da00bf",
            "metadata": {
                "vaultType": "base",
                "token": "0x186d0ba3dfc3386c464eecd96a61fbb1e2da00bf"
            }
        }
    }
}
